@@ -3,17 +3,32 @@
 #define ACLK_H
 
 #include "daemon/common.h"
+
+#ifdef ENABLE_ACLK
 #include "aclk_util.h"
 #include "aclk_rrdhost_state.h"
 
 // How many MQTT PUBACKs we need to get to consider connection
 // stable for the purposes of TBEB (truncated binary exponential backoff)
 #define ACLK_PUBACKS_CONN_STABLE 3
+#endif /* ENABLE_ACLK */
+
+extern int aclk_connected;
+extern int aclk_ctx_based;
+extern int aclk_disable_runtime;
+extern int aclk_stats_enabled;
+extern int aclk_kill_link;
+
+extern usec_t aclk_session_us;
+extern time_t aclk_session_sec;
 
 extern time_t aclk_block_until;
 
 extern int disconnect_req;
 
+extern int aclk_alert_reloaded;
+
+#ifdef ENABLE_ACLK
 void *aclk_main(void *ptr);
 
 extern netdata_mutex_t aclk_shared_state_mutex;
@@ -21,9 +36,6 @@ extern netdata_mutex_t aclk_shared_state_mutex;
 #define ACLK_SHARED_STATE_UNLOCK netdata_mutex_unlock(&aclk_shared_state_mutex)
 
 extern struct aclk_shared_state {
-    ACLK_AGENT_STATE agent_state;
-    time_t last_popcorn_interrupt;
-
     // To wait for `disconnect` message PUBACK
     // when shutting down
     // at the same time if > 0 we know link is
@@ -32,25 +44,16 @@ extern struct aclk_shared_state {
     int mqtt_shutdown_msg_rcvd;
 } aclk_shared_state;
 
-void aclk_alarm_reload(void);
-int aclk_update_alarm(RRDHOST *host, ALARM_ENTRY *ae);
-
-/* Informs ACLK about created/deleted chart
- * @param create 0 - if chart was deleted, other if chart created
- */
-int aclk_update_chart(RRDHOST *host, char *chart_name, int create);
-
-void aclk_add_collector(RRDHOST *host, const char *plugin_name, const char *module_name);
-void aclk_del_collector(RRDHOST *host, const char *plugin_name, const char *module_name);
-
-#ifdef ENABLE_NEW_CLOUD_PROTOCOL
 void aclk_host_state_update(RRDHOST *host, int cmd);
 void aclk_send_node_instances(void);
-#endif
 
 void aclk_send_bin_msg(char *msg, size_t msg_len, enum aclk_topics subtopic, const char *msgname);
 
-char *ng_aclk_state(void);
-char *ng_aclk_state_json(void);
+#endif /* ENABLE_ACLK */
+
+char *aclk_state(void);
+char *aclk_state_json(void);
+void add_aclk_host_labels(void);
+void aclk_queue_node_info(RRDHOST *host);
 
 #endif /* ACLK_H */
