@@ -195,11 +195,11 @@
 #define NETDATA_CHART_PRIO_ZFS_IO                     2700
 #define NETDATA_CHART_PRIO_ZFS_HITS                   2520
 #define NETDATA_CHART_PRIO_ZFS_DHITS                  2530
-#define NETDATA_CHART_PRIO_ZFS_DEMAND_DATA_HITS       2531
-#define NETDATA_CHART_PRIO_ZFS_PREFETCH_DATA_HITS     2532
-#define NETDATA_CHART_PRIO_ZFS_PHITS                  2540
-#define NETDATA_CHART_PRIO_ZFS_MHITS                  2550
-#define NETDATA_CHART_PRIO_ZFS_L2HITS                 2560
+#define NETDATA_CHART_PRIO_ZFS_DEMAND_DATA_HITS       2540
+#define NETDATA_CHART_PRIO_ZFS_PREFETCH_DATA_HITS     2550
+#define NETDATA_CHART_PRIO_ZFS_PHITS                  2560
+#define NETDATA_CHART_PRIO_ZFS_MHITS                  2570
+#define NETDATA_CHART_PRIO_ZFS_L2HITS                 2580
 #define NETDATA_CHART_PRIO_ZFS_LIST_HITS              2600
 #define NETDATA_CHART_PRIO_ZFS_HASH_ELEMENTS          2800
 #define NETDATA_CHART_PRIO_ZFS_HASH_CHAINS            2810
@@ -363,5 +363,30 @@
 #define NETDATA_CHART_PRIO_NETDATA_TIMEX            132030
 #define NETDATA_CHART_PRIO_NETDATA_TC_TIME          1000100
 
+// NETDATA ML CHARTS
+
+// [ml] charts
+#define ML_CHART_PRIO_DIMENSIONS        39181
+#define ML_CHART_PRIO_ANOMALY_RATE      39182
+#define ML_CHART_PRIO_DETECTOR_EVENTS   39183
+
+// [netdata.ml] charts
+#define NETDATA_ML_CHART_PRIO_MACHINE_LEARNING_STATUS 890001
+#define NETDATA_ML_CHART_PRIO_METRIC_TYPES            890002
+#define NETDATA_ML_CHART_PRIO_TRAINING_STATUS         890003
+
+#define NETDATA_ML_CHART_PRIO_PREDICTION_USAGE        890004
+#define NETDATA_ML_CHART_PRIO_TRAINING_USAGE          890005
+
+#define NETDATA_ML_CHART_PRIO_QUEUE_STATS             890006
+#define NETDATA_ML_CHART_PRIO_TRAINING_TIME_STATS     890007
+#define NETDATA_ML_CHART_PRIO_TRAINING_RESULTS        890008
+
+#define NETDATA_ML_CHART_FAMILY "machine learning"
+#define NETDATA_ML_PLUGIN "ml.plugin"
+#define NETDATA_ML_MODULE_TRAINING "training"
+#define NETDATA_ML_MODULE_DETECTION "detection"
+#define NETDATA_ML_MODULE_PREDICTION "prediction"
+
 
 #endif //NETDATA_ALL_H
