@@ -137,10 +137,7 @@ getIntegerOption(
   return ((int)intvalue);
 }
 
-static int reset_job_metrics(const char *name, void *entry, void *data) {
-    (void)name;
-    (void)data;
-
+static int reset_job_metrics(const DICTIONARY_ITEM *item __maybe_unused, void *entry, void *data __maybe_unused) {
     struct job_metrics *jm = (struct job_metrics *)entry;
 
     jm->is_collected = 0;
@@ -162,12 +159,12 @@ struct job_metrics *get_job_metrics(char *dest) {
         reset_job_metrics(NULL, &new_job_metrics, NULL);
         jm = dictionary_set(dict_dest_job_metrics, dest, &new_job_metrics, sizeof(struct job_metrics));
 
-        printf("CHART cups.job_num_%s '' 'Active job number of destination %s' jobs '%s' cups.job_num stacked %i %i\n", dest, dest, dest, netdata_priority++, netdata_update_every);
+        printf("CHART cups.job_num_%s '' 'Active jobs of %s' jobs '%s' cups.job_num stacked %i %i\n", dest, dest, dest, netdata_priority++, netdata_update_every);
         printf("DIMENSION pending '' absolute 1 1\n");
         printf("DIMENSION held '' absolute 1 1\n");
         printf("DIMENSION processing '' absolute 1 1\n");
 
-        printf("CHART cups.job_size_%s '' 'Active job size of destination %s' KB '%s' cups.job_size stacked %i %i\n", dest, dest, dest, netdata_priority++, netdata_update_every);
+        printf("CHART cups.job_size_%s '' 'Active jobs size of %s' KB '%s' cups.job_size stacked %i %i\n", dest, dest, dest, netdata_priority++, netdata_update_every);
         printf("DIMENSION pending '' absolute 1 1\n");
         printf("DIMENSION held '' absolute 1 1\n");
         printf("DIMENSION processing '' absolute 1 1\n");
@@ -175,8 +172,8 @@ struct job_metrics *get_job_metrics(char *dest) {
     return jm;
 }
 
-int collect_job_metrics(const char *name, void *entry, void *data) {
-    (void)data;
+int collect_job_metrics(const DICTIONARY_ITEM *item, void *entry, void *data __maybe_unused) {
+    const char *name = dictionary_acquired_item_name(item);
 
     struct job_metrics *jm = (struct job_metrics *)entry;
 
@@ -196,16 +193,16 @@ int collect_job_metrics(const char *name, void *entry, void *data) {
             "END\n",
             name, jm->size_pending, jm->size_held, jm->size_processing);
     } else {
-        printf("CHART cups.job_num_%s '' 'Active job number of destination %s' jobs '%s' cups.job_num stacked 1 %i 'obsolete'\n", name, name, name, netdata_update_every);
+        printf("CHART cups.job_num_%s '' 'Active jobs of %s' jobs '%s' cups.job_num stacked 1 %i 'obsolete'\n", name, name, name, netdata_update_every);
         printf("DIMENSION pending '' absolute 1 1\n");
         printf("DIMENSION held '' absolute 1 1\n");
         printf("DIMENSION processing '' absolute 1 1\n");
 
-        printf("CHART cups.job_size_%s '' 'Active job size of destination %s' KB '%s' cups.job_size stacked 1 %i 'obsolete'\n", name, name, name, netdata_update_every);
+        printf("CHART cups.job_size_%s '' 'Active jobs size of %s' KB '%s' cups.job_size stacked 1 %i 'obsolete'\n", name, name, name, netdata_update_every);
         printf("DIMENSION pending '' absolute 1 1\n");
         printf("DIMENSION held '' absolute 1 1\n");
         printf("DIMENSION processing '' absolute 1 1\n");
-        dictionary_del_having_write_lock(dict_dest_job_metrics, name);
+        dictionary_del(dict_dest_job_metrics, name);
     }
 
     return 0;
@@ -225,6 +222,7 @@ void reset_metrics() {
 }
 
 int main(int argc, char **argv) {
+    stderror = stderr;
     clocks_init();
 
     // ------------------------------------------------------------------------
@@ -243,7 +241,7 @@ int main(int argc, char **argv) {
 
     errno = 0;
 
-    dict_dest_job_metrics = dictionary_create(DICTIONARY_FLAG_SINGLE_THREADED);
+    dict_dest_job_metrics = dictionary_create(DICT_OPTION_SINGLE_THREADED);
 
     // ------------------------------------------------------------------------
     // the main loop
@@ -387,12 +385,12 @@ int main(int argc, char **argv) {
             printf("DIMENSION acceptingjobs '' absolute 1 1\n");
             printf("DIMENSION shared '' absolute 1 1\n");
 
-            printf("CHART cups.job_num '' 'Total active job number' jobs overview cups.job_num stacked 100002 %i\n", netdata_update_every);
+            printf("CHART cups.job_num '' 'Active jobs' jobs overview cups.job_num stacked 100002 %i\n", netdata_update_every);
             printf("DIMENSION pending '' absolute 1 1\n");
             printf("DIMENSION held '' absolute 1 1\n");
             printf("DIMENSION processing '' absolute 1 1\n");
 
-            printf("CHART cups.job_size '' 'Total active job size' KB overview cups.job_size stacked 100003 %i\n", netdata_update_every);
+            printf("CHART cups.job_size '' 'Active jobs size' KB overview cups.job_size stacked 100003 %i\n", netdata_update_every);
             printf("DIMENSION pending '' absolute 1 1\n");
             printf("DIMENSION held '' absolute 1 1\n");
             printf("DIMENSION processing '' absolute 1 1\n");
