@@ -7,7 +7,6 @@ static char *shm_dimension_name[NETDATA_SHM_END] = { "get", "at", "dt", "ctl" };
 static netdata_syscall_stat_t shm_aggregated_data[NETDATA_SHM_END];
 static netdata_publish_syscall_t shm_publish_aggregated[NETDATA_SHM_END];
 
-static int read_thread_closed = 1;
 netdata_publish_shm_t *shm_vector = NULL;
 
 static netdata_idx_t shm_hash_values[NETDATA_SHM_END];
@@ -35,12 +34,6 @@ static ebpf_local_maps_t shm_maps[] = {{.name = "tbl_pid_shm", .internal_input =
                                          .map_fd = ND_EBPF_MAP_FD_NOT_INITIALIZED},
                                         {.name = NULL, .internal_input = 0, .user_input = 0}};
 
-static struct bpf_link **probe_links = NULL;
-static struct bpf_object *objects = NULL;
-
-struct netdata_static_thread shm_threads = {"SHM KERNEL", NULL, NULL, 1,
-                                             NULL, NULL,  NULL};
-
 netdata_ebpf_targets_t shm_targets[] = { {.name = "shmget", .mode = EBPF_LOAD_TRAMPOLINE},
                                          {.name = "shmat", .mode = EBPF_LOAD_TRAMPOLINE},
                                          {.name = "shmdt", .mode = EBPF_LOAD_TRAMPOLINE},
@@ -86,6 +79,7 @@ static void ebpf_disable_probe(struct shm_bpf *obj)
     bpf_program__set_autoload(obj->progs.netdata_shmat_probe, false);
     bpf_program__set_autoload(obj->progs.netdata_shmdt_probe, false);
     bpf_program__set_autoload(obj->progs.netdata_shmctl_probe, false);
+    bpf_program__set_autoload(obj->progs.netdata_shm_release_task_probe, false);
 }
 
 /*
@@ -101,6 +95,7 @@ static void ebpf_disable_trampoline(struct shm_bpf *obj)
     bpf_program__set_autoload(obj->progs.netdata_shmat_fentry, false);
     bpf_program__set_autoload(obj->progs.netdata_shmdt_fentry, false);
     bpf_program__set_autoload(obj->progs.netdata_shmctl_fentry, false);
+    bpf_program__set_autoload(obj->progs.netdata_shm_release_task_fentry, false);
 }
 
 /**
@@ -133,6 +128,9 @@ static void ebpf_set_trampoline_target(struct shm_bpf *obj)
                             shm_targets[NETDATA_KEY_SHMCTL_CALL].name, running_on_kernel);
     bpf_program__set_attach_target(obj->progs.netdata_shmctl_fentry, 0,
                                    syscall);
+
+    bpf_program__set_attach_target(obj->progs.netdata_shm_release_task_fentry, 0,
+                                   EBPF_COMMON_FNCT_CLEAN_UP);
 }
 
 /**
@@ -180,6 +178,13 @@ static int ebpf_shm_attach_probe(struct shm_bpf *obj)
     if (ret)
         return -1;
 
+    obj->links.netdata_shm_release_task_probe = bpf_program__attach_kprobe(obj->progs.netdata_shm_release_task_probe,
+                                                                           false, EBPF_COMMON_FNCT_CLEAN_UP);
+    ret = (int)libbpf_get_error(obj->links.netdata_shm_release_task_probe);
+    if (ret)
+        return -1;
+
+
     return 0;
 }
 
@@ -195,6 +200,33 @@ static void ebpf_shm_set_hash_tables(struct shm_bpf *obj)
     shm_maps[NETDATA_SHM_GLOBAL_TABLE].map_fd = bpf_map__fd(obj->maps.tbl_shm);
 }
 
+/**
+ *  Disable Release Task
+ *
+ *  Disable release task when apps is not enabled.
+ *
+ *  @param obj is the main structure for bpf objects.
+ */
+static void ebpf_shm_disable_release_task(struct shm_bpf *obj)
+{
+    bpf_program__set_autoload(obj->progs.netdata_shm_release_task_probe, false);
+    bpf_program__set_autoload(obj->progs.netdata_shm_release_task_fentry, false);
+}
+
+/**
+ * Adjust Map Size
+ *
+ * Resize maps according input from users.
+ *
+ * @param obj is the main structure for bpf objects.
+ * @param em  structure with configuration
+ */
+static void ebpf_shm_adjust_map_size(struct shm_bpf *obj, ebpf_module_t *em)
+{
+    ebpf_update_map_size(obj->maps.tbl_pid_shm, &shm_maps[NETDATA_PID_SHM_TABLE],
+                         em, bpf_map__name(obj->maps.tbl_pid_shm));
+}
+
 /**
  * Load and attach
  *
@@ -203,7 +235,7 @@ static void ebpf_shm_set_hash_tables(struct shm_bpf *obj)
  * @param obj is the main structure for bpf objects.
  * @param em  structure with configuration
  *
- * @return it returns 0 on succes and -1 otherwise
+ * @return it returns 0 on success and -1 otherwise
  */
 static inline int ebpf_shm_load_and_attach(struct shm_bpf *obj, ebpf_module_t *em)
 {
@@ -224,6 +256,10 @@ static inline int ebpf_shm_load_and_attach(struct shm_bpf *obj, ebpf_module_t *e
         ebpf_disable_trampoline(obj);
     }
 
+    ebpf_shm_adjust_map_size(obj, em);
+    if (!em->apps_charts && !em->cgroup_charts)
+        ebpf_shm_disable_release_task(obj);
+
     int ret = shm_bpf__load(obj);
     if (!ret) {
         if (test != EBPF_LOAD_PROBE && test != EBPF_LOAD_RETPROBE)
@@ -243,55 +279,44 @@ static inline int ebpf_shm_load_and_attach(struct shm_bpf *obj, ebpf_module_t *e
  *****************************************************************/
 
 /**
- * Clean shm structure
- */
-void clean_shm_pid_structures() {
-    struct pid_stat *pids = root_of_pids;
-    while (pids) {
-        freez(shm_pid[pids->pid]);
-
-        pids = pids->next;
-    }
-}
-
-/**
- * Clean up the main thread.
+ * SHM Free
+ *
+ * Cleanup variables after child threads to stop
  *
  * @param ptr thread data.
  */
-static void ebpf_shm_cleanup(void *ptr)
+static void ebpf_shm_free(ebpf_module_t *em)
 {
-    ebpf_module_t *em = (ebpf_module_t *)ptr;
-    if (!em->enabled) {
-        return;
-    }
-
-    heartbeat_t hb;
-    heartbeat_init(&hb);
-    uint32_t tick = 2 * USEC_PER_MS;
-    while (!read_thread_closed) {
-        usec_t dt = heartbeat_next(&hb, tick);
-        UNUSED(dt);
-    }
+    pthread_mutex_lock(&ebpf_exit_cleanup);
+    em->thread->enabled = NETDATA_THREAD_EBPF_STOPPING;
+    pthread_mutex_unlock(&ebpf_exit_cleanup);
 
     ebpf_cleanup_publish_syscall(shm_publish_aggregated);
 
     freez(shm_vector);
     freez(shm_values);
 
-    if (probe_links) {
-        struct bpf_program *prog;
-        size_t i = 0 ;
-        bpf_object__for_each_program(prog, objects) {
-            bpf_link__destroy(probe_links[i]);
-            i++;
-        }
-        bpf_object__close(objects);
-    }
 #ifdef LIBBPF_MAJOR_VERSION
-    else if (bpf_obj)
+    if (bpf_obj)
         shm_bpf__destroy(bpf_obj);
 #endif
+
+    pthread_mutex_lock(&ebpf_exit_cleanup);
+    em->thread->enabled = NETDATA_THREAD_EBPF_STOPPED;
+    pthread_mutex_unlock(&ebpf_exit_cleanup);
+}
+
+/**
+ * SHM Exit
+ *
+ * Cancel child thread.
+ *
+ * @param ptr thread data.
+ */
+static void ebpf_shm_exit(void *ptr)
+{
+    ebpf_module_t *em = (ebpf_module_t *)ptr;
+    ebpf_shm_free(em);
 }
 
 /*****************************************************************
@@ -439,7 +464,7 @@ static void shm_send_global()
  *
  * Read the table with number of calls for all functions
  */
-static void read_global_table()
+static void ebpf_shm_read_global_table()
 {
     netdata_idx_t *stored = shm_values;
     netdata_idx_t *val = shm_hash_values;
@@ -459,32 +484,6 @@ static void read_global_table()
     }
 }
 
-/**
- * Shared memory reader thread.
- *
- * @param ptr It is a NULL value for this thread.
- * @return It always returns NULL.
- */
-void *ebpf_shm_read_hash(void *ptr)
-{
-    read_thread_closed = 0;
-
-    heartbeat_t hb;
-    heartbeat_init(&hb);
-
-    ebpf_module_t *em = (ebpf_module_t *)ptr;
-    usec_t step = NETDATA_SHM_SLEEP_MS * em->update_every;
-    while (!close_ebpf_plugin) {
-        usec_t dt = heartbeat_next(&hb, step);
-        (void)dt;
-
-        read_global_table();
-    }
-
-    read_thread_closed = 1;
-    return NULL;
-}
-
 /**
  * Sum values for all targets.
  */
@@ -728,20 +727,15 @@ static void ebpf_create_systemd_shm_charts(int update_every)
  * Send Systemd charts
  *
  * Send collected data to Netdata.
- *
- * @return It returns the status for chart creation, if it is necessary to remove a specific dimension, zero is returned
- *         otherwise function returns 1 to avoid chart recreation
  */
-static int ebpf_send_systemd_shm_charts()
+static void ebpf_send_systemd_shm_charts()
 {
-    int ret = 1;
     ebpf_cgroup_target_t *ect;
     write_begin_chart(NETDATA_SERVICE_FAMILY, NETDATA_SHMGET_CHART);
     for (ect = ebpf_cgroup_pids; ect; ect = ect->next) {
         if (unlikely(ect->systemd) && unlikely(ect->updated)) {
             write_chart_dimension(ect->name, (long long)ect->publish_shm.get);
-        } else
-            ret = 0;
+        }
     }
     write_end_chart();
 
@@ -768,8 +762,6 @@ static int ebpf_send_systemd_shm_charts()
         }
     }
     write_end_chart();
-
-    return ret;
 }
 
 /*
@@ -817,13 +809,11 @@ void ebpf_shm_send_cgroup_data(int update_every)
 
     int has_systemd = shm_ebpf_cgroup.header->systemd_enabled;
     if (has_systemd) {
-        static int systemd_charts = 0;
-        if (!systemd_charts) {
+        if (send_cgroup_chart) {
             ebpf_create_systemd_shm_charts(update_every);
-            systemd_charts = 1;
         }
 
-        systemd_charts = ebpf_send_systemd_shm_charts();
+        ebpf_send_systemd_shm_charts();
     }
 
     for (ect = ebpf_cgroup_pids; ect ; ect = ect->next) {
@@ -853,50 +843,41 @@ void ebpf_shm_send_cgroup_data(int update_every)
 */
 static void shm_collector(ebpf_module_t *em)
 {
-    shm_threads.thread = mallocz(sizeof(netdata_thread_t));
-    shm_threads.start_routine = ebpf_shm_read_hash;
-
-    netdata_thread_create(
-        shm_threads.thread,
-        shm_threads.name,
-        NETDATA_THREAD_OPTION_JOINABLE,
-        ebpf_shm_read_hash,
-        em
-    );
-
-    int apps = em->apps_charts;
     int cgroups = em->cgroup_charts;
     int update_every = em->update_every;
+    heartbeat_t hb;
+    heartbeat_init(&hb);
     int counter = update_every - 1;
-    while (!close_ebpf_plugin) {
+    while (!ebpf_exit_plugin) {
+        (void)heartbeat_next(&hb, USEC_PER_SEC);
+        if (ebpf_exit_plugin || ++counter != update_every)
+            continue;
+
+        counter = 0;
+        netdata_apps_integration_flags_t apps = em->apps_charts;
+        ebpf_shm_read_global_table();
         pthread_mutex_lock(&collect_data_mutex);
-        pthread_cond_wait(&collect_data_cond_var, &collect_data_mutex);
-
-        if (++counter == update_every) {
-            counter = 0;
-            if (apps) {
-                read_apps_table();
-            }
-
-            if (cgroups) {
-                ebpf_update_shm_cgroup();
-            }
-
-            pthread_mutex_lock(&lock);
-
-            shm_send_global();
-
-            if (apps) {
-                ebpf_shm_send_apps_data(apps_groups_root_target);
-            }
-
-            if (cgroups) {
-                ebpf_shm_send_cgroup_data(update_every);
-            }
-
-            pthread_mutex_unlock(&lock);
+        if (apps) {
+            read_apps_table();
         }
 
+        if (cgroups) {
+            ebpf_update_shm_cgroup();
+        }
+
+        pthread_mutex_lock(&lock);
+
+        shm_send_global();
+
+        if (apps & NETDATA_EBPF_APPS_FLAG_CHART_CREATED) {
+            ebpf_shm_send_apps_data(apps_groups_root_target);
+        }
+
+        if (cgroups) {
+            ebpf_shm_send_cgroup_data(update_every);
+        }
+
+        pthread_mutex_unlock(&lock);
         pthread_mutex_unlock(&collect_data_mutex);
     }
 }
@@ -950,6 +931,8 @@ void ebpf_shm_create_apps_charts(struct ebpf_module *em, void *ptr)
                                20194,
                                ebpf_algorithms[NETDATA_EBPF_INCREMENTAL_IDX],
                                root, em->update_every, NETDATA_EBPF_MODULE_NAME_SHM);
+
+    em->apps_charts |= NETDATA_EBPF_APPS_FLAG_CHART_CREATED;
 }
 
 /**
@@ -1013,9 +996,11 @@ static void ebpf_create_shm_charts(int update_every)
 static int ebpf_shm_load_bpf(ebpf_module_t *em)
 {
     int ret = 0;
-    if (em->load == EBPF_LOAD_LEGACY) {
-        probe_links = ebpf_load_program(ebpf_plugin_dir, em, running_on_kernel, isrh, &objects);
-        if (!probe_links) {
+
+    ebpf_adjust_apps_cgroup(em, em->targets[NETDATA_KEY_SHMGET_CALL].mode);
+    if (em->load & EBPF_LOAD_LEGACY) {
+        em->probe_links = ebpf_load_program(ebpf_plugin_dir, em, running_on_kernel, isrh, &em->objects);
+        if (!em->probe_links) {
             em->enabled = CONFIG_BOOLEAN_NO;
             ret = -1;
         }
@@ -1045,22 +1030,18 @@ static int ebpf_shm_load_bpf(ebpf_module_t *em)
  */
 void *ebpf_shm_thread(void *ptr)
 {
-    netdata_thread_cleanup_push(ebpf_shm_cleanup, ptr);
+    netdata_thread_cleanup_push(ebpf_shm_exit, ptr);
 
     ebpf_module_t *em = (ebpf_module_t *)ptr;
     em->maps = shm_maps;
 
     ebpf_update_pid_table(&shm_maps[NETDATA_PID_SHM_TABLE], em);
 
-    if (!em->enabled) {
-        goto endshm;
-    }
-
 #ifdef LIBBPF_MAJOR_VERSION
     ebpf_adjust_thread_load(em, default_btf);
 #endif
     if (ebpf_shm_load_bpf(em)) {
-        em->enabled = CONFIG_BOOLEAN_NO;
+        em->thread->enabled = NETDATA_THREAD_EBPF_STOPPED;
         goto endshm;
     }
 
@@ -1089,8 +1070,7 @@ void *ebpf_shm_thread(void *ptr)
     shm_collector(em);
 
 endshm:
-    if (!em->enabled)
-        ebpf_update_disabled_plugin_stats(em);
+    ebpf_update_disabled_plugin_stats(em);
 
     netdata_thread_cleanup_pop(1);
     return NULL;
