@@ -244,6 +244,7 @@ typedef struct ebpf_network_viewer_hostname_list {
 
 #define NETDATA_NV_CAP_VALUE 50L
 typedef struct ebpf_network_viewer_options {
+    uint32_t enabled;
     uint32_t max_dim;   // Store value read from 'maximum dimensions'
 
     uint32_t hostname_resolution_enabled;
@@ -279,7 +280,7 @@ typedef struct netdata_socket {
     uint32_t retransmit; // It is never used with UDP
     uint16_t protocol;
     uint16_t reserved;
-} netdata_socket_t __attribute__((__aligned__(8)));
+} netdata_socket_t;
 
 typedef struct netdata_plot_values {
     // Values used in the previous iteration
@@ -307,7 +308,7 @@ typedef struct netdata_socket_idx {
     uint16_t sport;
     union netdata_ip_t daddr;
     uint16_t dport;
-} netdata_socket_idx_t __attribute__((__aligned__(8)));
+} netdata_socket_idx_t;
 
 // Next values were defined according getnameinfo(3)
 #define NETDATA_MAX_NETWORK_COMBINED_LENGTH 1018
@@ -356,13 +357,12 @@ typedef struct netdata_vector_plot {
 
 } netdata_vector_plot_t;
 
-extern void clean_port_structure(ebpf_network_viewer_port_list_t **clean);
+void clean_port_structure(ebpf_network_viewer_port_list_t **clean);
 extern ebpf_network_viewer_port_list_t *listen_ports;
-extern void update_listen_table(uint16_t value, uint16_t proto, netdata_passive_connection_t *values);
-extern void parse_network_viewer_section(struct config *cfg);
-extern void fill_ip_list(ebpf_network_viewer_ip_list_t **out, ebpf_network_viewer_ip_list_t *in, char *table);
-extern void parse_service_name_section(struct config *cfg);
-extern void clean_socket_apps_structures();
+void update_listen_table(uint16_t value, uint16_t proto, netdata_passive_connection_t *values);
+void parse_network_viewer_section(struct config *cfg);
+void ebpf_fill_ip_list(ebpf_network_viewer_ip_list_t **out, ebpf_network_viewer_ip_list_t *in, char *table);
+void parse_service_name_section(struct config *cfg);
 
 extern ebpf_socket_publish_apps_t **socket_bandwidth_curr;
 extern struct config socket_config;
