@@ -7,19 +7,41 @@
 
 #define STATE_LENGTH_MAX 32
 
-// As defined in https://www.kernel.org/doc/Documentation/ABI/testing/sysfs-class-net
-const char *operstate_names[] = { "unknown", "notpresent", "down", "lowerlayerdown", "testing", "dormant", "up" };
+#define READ_RETRY_PERIOD 60 // seconds
+
+enum {
+    NETDEV_DUPLEX_UNKNOWN,
+    NETDEV_DUPLEX_HALF,
+    NETDEV_DUPLEX_FULL
+};
+
+enum {
+    NETDEV_OPERSTATE_UNKNOWN,
+    NETDEV_OPERSTATE_NOTPRESENT,
+    NETDEV_OPERSTATE_DOWN,
+    NETDEV_OPERSTATE_LOWERLAYERDOWN,
+    NETDEV_OPERSTATE_TESTING,
+    NETDEV_OPERSTATE_DORMANT,
+    NETDEV_OPERSTATE_UP
+};
 
 static inline int get_operstate(char *operstate)
 {
-    int i;
+    // As defined in https://www.kernel.org/doc/Documentation/ABI/testing/sysfs-class-net
+    if (!strcmp(operstate, "up"))
+        return NETDEV_OPERSTATE_UP;
+    if (!strcmp(operstate, "down"))
+        return NETDEV_OPERSTATE_DOWN;
+    if (!strcmp(operstate, "notpresent"))
+        return NETDEV_OPERSTATE_NOTPRESENT;
+    if (!strcmp(operstate, "lowerlayerdown"))
+        return NETDEV_OPERSTATE_LOWERLAYERDOWN;
+    if (!strcmp(operstate, "testing"))
+        return NETDEV_OPERSTATE_TESTING;
+    if (!strcmp(operstate, "dormant"))
+        return NETDEV_OPERSTATE_DORMANT;
 
-    for (i = 0; i < (int) (sizeof(operstate_names) / sizeof(char *)); i++) {
-        if (!strcmp(operstate, operstate_names[i])) {
-            return i;
-        }
-    }
-    return 0;
+    return NETDEV_OPERSTATE_UNKNOWN;
 }
 
 // ----------------------------------------------------------------------------
@@ -35,6 +57,15 @@ static struct netdev {
     int configured;
     int enabled;
     int updated;
+    
+    int carrier_file_exists;
+    time_t carrier_file_lost_time;
+
+    int duplex_file_exists;
+    time_t duplex_file_lost_time;
+
+    int speed_file_exists;
+    time_t speed_file_lost_time;
 
     int do_bandwidth;
     int do_packets;
@@ -90,7 +121,7 @@ static struct netdev {
 
     const char *chart_family;
 
-    struct label *chart_labels;
+    DICTIONARY *chart_labels;
 
     int flipped;
     unsigned long priority;
@@ -153,13 +184,22 @@ static struct netdev {
     RRDDIM *rd_tcompressed;
 
     RRDDIM *rd_speed;
-    RRDDIM *rd_duplex;
-    RRDDIM *rd_operstate;
-    RRDDIM *rd_carrier;
+    RRDDIM *rd_duplex_full;
+    RRDDIM *rd_duplex_half;
+    RRDDIM *rd_duplex_unknown;
+    RRDDIM *rd_operstate_unknown;
+    RRDDIM *rd_operstate_notpresent;
+    RRDDIM *rd_operstate_down;
+    RRDDIM *rd_operstate_lowerlayerdown;
+    RRDDIM *rd_operstate_testing;
+    RRDDIM *rd_operstate_dormant;
+    RRDDIM *rd_operstate_up;
+    RRDDIM *rd_carrier_up;
+    RRDDIM *rd_carrier_down;
     RRDDIM *rd_mtu;
 
     char *filename_speed;
-    RRDSETVAR *chart_var_speed;
+    const RRDSETVAR_ACQUIRED *chart_var_speed;
 
     char *filename_duplex;
     char *filename_operstate;
@@ -219,11 +259,21 @@ static void netdev_charts_release(struct netdev *d) {
     d->rd_tcompressed = NULL;
 
     d->rd_speed       = NULL;
-    d->rd_duplex      = NULL;
-    d->rd_operstate   = NULL;
-    d->rd_carrier     = NULL;
+    d->rd_duplex_full = NULL;
+    d->rd_duplex_half = NULL;
+    d->rd_duplex_unknown = NULL;
+    d->rd_carrier_up = NULL;
+    d->rd_carrier_down = NULL;
     d->rd_mtu         = NULL;
 
+    d->rd_operstate_unknown = NULL;
+    d->rd_operstate_notpresent = NULL;
+    d->rd_operstate_down = NULL;
+    d->rd_operstate_lowerlayerdown = NULL;
+    d->rd_operstate_testing = NULL;
+    d->rd_operstate_dormant = NULL;
+    d->rd_operstate_up = NULL;
+
     d->chart_var_speed     = NULL;
 }
 
@@ -273,7 +323,7 @@ static void netdev_free_chart_strings(struct netdev *d) {
 static void netdev_free(struct netdev *d) {
     netdev_charts_release(d);
     netdev_free_chart_strings(d);
-    free_label_list(d->chart_labels);
+    rrdlabels_destroy(d->chart_labels);
 
     freez((void *)d->name);
     freez((void *)d->filename_speed);
@@ -294,8 +344,9 @@ static struct netdev_rename {
 
     const char *container_device;
     const char *container_name;
+    const char *ctx_prefix;
 
-    struct label *chart_labels;
+    DICTIONARY *chart_labels;
 
     int processed;
 
@@ -317,7 +368,11 @@ static struct netdev_rename *netdev_rename_find(const char *host_device, uint32_
 
 // other threads can call this function to register a rename to a netdev
 void netdev_rename_device_add(
-    const char *host_device, const char *container_device, const char *container_name, struct label *labels)
+    const char *host_device,
+    const char *container_device,
+    const char *container_name,
+    DICTIONARY *labels,
+    const char *ctx_prefix)
 {
     netdata_mutex_lock(&netdev_rename_mutex);
 
@@ -328,13 +383,15 @@ void netdev_rename_device_add(
         r->host_device      = strdupz(host_device);
         r->container_device = strdupz(container_device);
         r->container_name   = strdupz(container_name);
-        update_label_list(&r->chart_labels, labels);
+        r->ctx_prefix       = strdupz(ctx_prefix);
+        r->chart_labels     = rrdlabels_create();
+        rrdlabels_migrate_to_these(r->chart_labels, labels);
         r->hash             = hash;
         r->next             = netdev_rename_root;
         r->processed        = 0;
         netdev_rename_root  = r;
         netdev_pending_renames++;
-        info("CGROUP: registered network interface rename for '%s' as '%s' under '%s'", r->host_device, r->container_device, r->container_name);
+        collector_info("CGROUP: registered network interface rename for '%s' as '%s' under '%s'", r->host_device, r->container_device, r->container_name);
     }
     else {
         if(strcmp(r->container_device, container_device) != 0 || strcmp(r->container_name, container_name) != 0) {
@@ -344,11 +401,11 @@ void netdev_rename_device_add(
             r->container_device = strdupz(container_device);
             r->container_name   = strdupz(container_name);
 
-            update_label_list(&r->chart_labels, labels);
+            rrdlabels_migrate_to_these(r->chart_labels, labels);
             
             r->processed        = 0;
             netdev_pending_renames++;
-            info("CGROUP: altered network interface rename for '%s' as '%s' under '%s'", r->host_device, r->container_device, r->container_name);
+            collector_info("CGROUP: altered network interface rename for '%s' as '%s' under '%s'", r->host_device, r->container_device, r->container_name);
         }
     }
 
@@ -372,12 +429,13 @@ void netdev_rename_device_del(const char *host_device) {
             if(!r->processed)
                 netdev_pending_renames--;
 
-            info("CGROUP: unregistered network interface rename for '%s' as '%s' under '%s'", r->host_device, r->container_device, r->container_name);
+            collector_info("CGROUP: unregistered network interface rename for '%s' as '%s' under '%s'", r->host_device, r->container_device, r->container_name);
 
             freez((void *) r->host_device);
             freez((void *) r->container_name);
             freez((void *) r->container_device);
-            free_label_list(r->chart_labels);
+            freez((void *) r->ctx_prefix);
+            rrdlabels_destroy(r->chart_labels);
             freez((void *) r);
             break;
         }
@@ -387,7 +445,7 @@ void netdev_rename_device_del(const char *host_device) {
 }
 
 static inline void netdev_rename_cgroup(struct netdev *d, struct netdev_rename *r) {
-    info("CGROUP: renaming network interface '%s' as '%s' under '%s'", r->host_device, r->container_device, r->container_name);
+    collector_info("CGROUP: renaming network interface '%s' as '%s' under '%s'", r->host_device, r->container_device, r->container_name);
 
     netdev_charts_release(d);
     netdev_free_chart_strings(d);
@@ -433,23 +491,34 @@ static inline void netdev_rename_cgroup(struct netdev *d, struct netdev_rename *
     snprintfz(buffer, RRD_ID_LENGTH_MAX, "net_mtu_%s", r->container_device);
     d->chart_id_net_mtu        = strdupz(buffer);
 
-    d->chart_ctx_net_bytes      = strdupz("cgroup.net_net");
-    d->chart_ctx_net_compressed = strdupz("cgroup.net_compressed");
-    d->chart_ctx_net_drops      = strdupz("cgroup.net_drops");
-    d->chart_ctx_net_errors     = strdupz("cgroup.net_errors");
-    d->chart_ctx_net_events     = strdupz("cgroup.net_events");
-    d->chart_ctx_net_fifo       = strdupz("cgroup.net_fifo");
-    d->chart_ctx_net_packets    = strdupz("cgroup.net_packets");
-    d->chart_ctx_net_speed      = strdupz("cgroup.net_speed");
-    d->chart_ctx_net_duplex     = strdupz("cgroup.net_duplex");
-    d->chart_ctx_net_operstate  = strdupz("cgroup.net_operstate");
-    d->chart_ctx_net_carrier    = strdupz("cgroup.net_carrier");
-    d->chart_ctx_net_mtu        = strdupz("cgroup.net_mtu");
+    snprintfz(buffer, RRD_ID_LENGTH_MAX, "%scgroup.net_net", r->ctx_prefix);
+    d->chart_ctx_net_bytes      = strdupz(buffer);
+    snprintfz(buffer, RRD_ID_LENGTH_MAX, "%scgroup.net_compressed", r->ctx_prefix);
+    d->chart_ctx_net_compressed = strdupz(buffer);
+    snprintfz(buffer, RRD_ID_LENGTH_MAX, "%scgroup.net_drops", r->ctx_prefix);
+    d->chart_ctx_net_drops      = strdupz(buffer);
+    snprintfz(buffer, RRD_ID_LENGTH_MAX, "%scgroup.net_errors", r->ctx_prefix);
+    d->chart_ctx_net_errors     = strdupz(buffer);
+    snprintfz(buffer, RRD_ID_LENGTH_MAX, "%scgroup.net_events", r->ctx_prefix);
+    d->chart_ctx_net_events     = strdupz(buffer);
+    snprintfz(buffer, RRD_ID_LENGTH_MAX, "%scgroup.net_fifo", r->ctx_prefix);
+    d->chart_ctx_net_fifo       = strdupz(buffer);
+    snprintfz(buffer, RRD_ID_LENGTH_MAX, "%scgroup.net_packets", r->ctx_prefix);
+    d->chart_ctx_net_packets    = strdupz(buffer);
+    snprintfz(buffer, RRD_ID_LENGTH_MAX, "%scgroup.net_speed", r->ctx_prefix);
+    d->chart_ctx_net_speed      = strdupz(buffer);
+    snprintfz(buffer, RRD_ID_LENGTH_MAX, "%scgroup.net_duplex", r->ctx_prefix);
+    d->chart_ctx_net_duplex     = strdupz(buffer);
+    snprintfz(buffer, RRD_ID_LENGTH_MAX, "%scgroup.net_operstate", r->ctx_prefix);
+    d->chart_ctx_net_operstate  = strdupz(buffer);
+    snprintfz(buffer, RRD_ID_LENGTH_MAX, "%scgroup.net_carrier", r->ctx_prefix);
+    d->chart_ctx_net_carrier    = strdupz(buffer);
+    snprintfz(buffer, RRD_ID_LENGTH_MAX, "%scgroup.net_mtu", r->ctx_prefix);
+    d->chart_ctx_net_mtu        = strdupz(buffer);
 
-    snprintfz(buffer, RRD_ID_LENGTH_MAX, "net %s", r->container_device);
-    d->chart_family = strdupz(buffer);
+    d->chart_family = strdupz("net");
 
-    update_label_list(&d->chart_labels, r->chart_labels);
+    rrdlabels_copy(d->chart_labels, r->chart_labels);
 
     d->priority = NETDATA_CHART_PRIO_CGROUP_NET_IFACE;
     d->flipped = 1;
@@ -491,7 +560,7 @@ static void netdev_cleanup() {
     struct netdev *d = netdev_root, *last = NULL;
     while(d) {
         if(unlikely(!d->updated)) {
-            // info("Removing network device '%s', linked after '%s'", d->name, last?last->name:"ROOT");
+            // collector_info("Removing network device '%s', linked after '%s'", d->name, last?last->name:"ROOT");
 
             if(netdev_last_used == d)
                 netdev_last_used = last;
@@ -542,6 +611,7 @@ static struct netdev *get_netdev(const char *name) {
     d->name = strdupz(name);
     d->hash = simple_hash(d->name);
     d->len = strlen(d->name);
+    d->chart_labels = rrdlabels_create();
 
     d->chart_type_net_bytes      = strdupz("net");
     d->chart_type_net_compressed = strdupz("net_compressed");
@@ -702,11 +772,15 @@ int do_proc_net_dev(int update_every, usec_t dt) {
             char buffer[FILENAME_MAX + 1];
 
             snprintfz(buffer, FILENAME_MAX, path_to_sys_devices_virtual_net, d->name);
-            if(likely(access(buffer, R_OK) == 0)) {
+            if (likely(access(buffer, R_OK) == 0)) {
                 d->virtual = 1;
+                rrdlabels_add(d->chart_labels, "interface_type", "virtual", RRDLABEL_SRC_AUTO|RRDLABEL_FLAG_PERMANENT);
             }
-            else
+            else {
                 d->virtual = 0;
+                rrdlabels_add(d->chart_labels, "interface_type", "real", RRDLABEL_SRC_AUTO|RRDLABEL_FLAG_PERMANENT);
+            }
+            rrdlabels_add(d->chart_labels, "device", name, RRDLABEL_SRC_AUTO|RRDLABEL_FLAG_PERMANENT);
 
             if(likely(!d->virtual)) {
                 // set the filename to get the interface speed
@@ -795,39 +869,57 @@ int do_proc_net_dev(int update_every, usec_t dt) {
         if ((d->do_carrier != CONFIG_BOOLEAN_NO ||
              d->do_duplex != CONFIG_BOOLEAN_NO ||
              d->do_speed != CONFIG_BOOLEAN_NO) &&
-            d->filename_carrier) {
+             d->filename_carrier &&
+            (d->carrier_file_exists ||
+             now_monotonic_sec() - d->carrier_file_lost_time > READ_RETRY_PERIOD)) {
             if (read_single_number_file(d->filename_carrier, &d->carrier)) {
-                error("Cannot refresh interface %s carrier state by reading '%s'. Stop updating it.", d->name, d->filename_carrier);
-                freez(d->filename_carrier);
-                d->filename_carrier = NULL;
+                if (d->carrier_file_exists)
+                    collector_error(
+                        "Cannot refresh interface %s carrier state by reading '%s'. Next update is in %d seconds.",
+                        d->name,
+                        d->filename_carrier,
+                        READ_RETRY_PERIOD);
+                d->carrier_file_exists = 0;
+                d->carrier_file_lost_time = now_monotonic_sec();
+            } else {
+                d->carrier_file_exists = 1;
+                d->carrier_file_lost_time = 0;
             }
         }
 
-        if (d->do_duplex != CONFIG_BOOLEAN_NO && d->filename_duplex && (d->carrier || !d->filename_carrier)) {
+        if (d->do_duplex != CONFIG_BOOLEAN_NO &&
+            d->filename_duplex &&
+            (d->carrier || d->carrier_file_exists) &&
+            (d->duplex_file_exists ||
+             now_monotonic_sec() - d->duplex_file_lost_time > READ_RETRY_PERIOD)) {
             char buffer[STATE_LENGTH_MAX + 1];
 
             if (read_file(d->filename_duplex, buffer, STATE_LENGTH_MAX)) {
-                error("Cannot refresh interface %s duplex state by reading '%s'. I will stop updating it.", d->name, d->filename_duplex);
-                freez(d->filename_duplex);
-                d->filename_duplex = NULL;
+                if (d->duplex_file_exists)
+                    collector_error("Cannot refresh interface %s duplex state by reading '%s'.", d->name, d->filename_duplex);
+                d->duplex_file_exists = 0;
+                d->duplex_file_lost_time = now_monotonic_sec();
+                d->duplex = NETDEV_DUPLEX_UNKNOWN;
             } else {
                 // values can be unknown, half or full -- just check the first letter for speed
                 if (buffer[0] == 'f')
-                    d->duplex = 2;
+                    d->duplex = NETDEV_DUPLEX_FULL;
                 else if (buffer[0] == 'h')
-                    d->duplex = 1;
+                    d->duplex = NETDEV_DUPLEX_HALF;
                 else
-                    d->duplex = 0;
+                    d->duplex = NETDEV_DUPLEX_UNKNOWN;
+                d->duplex_file_exists = 1;
+                d->duplex_file_lost_time = 0;
             }
         } else {
-            d->duplex = 0;
+            d->duplex = NETDEV_DUPLEX_UNKNOWN;
         }
 
         if(d->do_operstate != CONFIG_BOOLEAN_NO && d->filename_operstate) {
             char buffer[STATE_LENGTH_MAX + 1], *trimmed_buffer;
 
             if (read_file(d->filename_operstate, buffer, STATE_LENGTH_MAX)) {
-                error(
+                collector_error(
                     "Cannot refresh %s operstate by reading '%s'. Will not update its status anymore.",
                     d->name, d->filename_operstate);
                 freez(d->filename_operstate);
@@ -840,13 +932,14 @@ int do_proc_net_dev(int update_every, usec_t dt) {
 
         if (d->do_mtu != CONFIG_BOOLEAN_NO && d->filename_mtu) {
             if (read_single_number_file(d->filename_mtu, &d->mtu)) {
-                error("Cannot refresh mtu for interface %s by reading '%s'. Stop updating it.", d->name, d->filename_mtu);
+                collector_error(
+                    "Cannot refresh mtu for interface %s by reading '%s'. Stop updating it.", d->name, d->filename_mtu);
                 freez(d->filename_mtu);
                 d->filename_mtu = NULL;
             }
         }
 
-        //info("PROC_NET_DEV: %s speed %zu, bytes %zu/%zu, packets %zu/%zu/%zu, errors %zu/%zu, drops %zu/%zu, fifo %zu/%zu, compressed %zu/%zu, rframe %zu, tcollisions %zu, tcarrier %zu"
+        //collector_info("PROC_NET_DEV: %s speed %zu, bytes %zu/%zu, packets %zu/%zu/%zu, errors %zu/%zu, drops %zu/%zu, fifo %zu/%zu, compressed %zu/%zu, rframe %zu, tcollisions %zu, tcarrier %zu"
         //        , d->name, d->speed
         //        , d->rbytes, d->tbytes
         //        , d->rpackets, d->tpackets, d->rmulticast
@@ -857,8 +950,6 @@ int do_proc_net_dev(int update_every, usec_t dt) {
         //        , d->rframe, d->tcollisions, d->tcarrier
         //        );
 
-        // --------------------------------------------------------------------
-
         if(unlikely(d->do_bandwidth == CONFIG_BOOLEAN_AUTO &&
                     (d->rbytes || d->tbytes || netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES)))
             d->do_bandwidth = CONFIG_BOOLEAN_YES;
@@ -881,7 +972,7 @@ int do_proc_net_dev(int update_every, usec_t dt) {
                         , RRDSET_TYPE_AREA
                 );
 
-                rrdset_update_labels(d->st_bandwidth, d->chart_labels);
+                rrdset_update_rrdlabels(d->st_bandwidth, d->chart_labels);
 
                 d->rd_rbytes = rrddim_add(d->st_bandwidth, "received", NULL,  8, BITS_IN_A_KILOBIT, RRD_ALGORITHM_INCREMENTAL);
                 d->rd_tbytes = rrddim_add(d->st_bandwidth, "sent",     NULL, -8, BITS_IN_A_KILOBIT, RRD_ALGORITHM_INCREMENTAL);
@@ -894,7 +985,6 @@ int do_proc_net_dev(int update_every, usec_t dt) {
                     d->rd_tbytes = td;
                 }
             }
-            else rrdset_next(d->st_bandwidth);
 
             rrddim_set_by_pointer(d->st_bandwidth, d->rd_rbytes, (collected_number)d->rbytes);
             rrddim_set_by_pointer(d->st_bandwidth, d->rd_tbytes, (collected_number)d->tbytes);
@@ -903,31 +993,34 @@ int do_proc_net_dev(int update_every, usec_t dt) {
             // update the interface speed
             if(d->filename_speed) {
                 if(unlikely(!d->chart_var_speed)) {
-                    d->chart_var_speed = rrdsetvar_custom_chart_variable_create(d->st_bandwidth, "nic_speed_max");
+                    d->chart_var_speed =
+                        rrdsetvar_custom_chart_variable_add_and_acquire(d->st_bandwidth, "nic_speed_max");
                     if(!d->chart_var_speed) {
-                        error("Cannot create interface %s chart variable 'nic_speed_max'. Will not update its speed anymore.", d->name);
+                        collector_error(
+                            "Cannot create interface %s chart variable 'nic_speed_max'. Will not update its speed anymore.",
+                            d->name);
                         freez(d->filename_speed);
                         d->filename_speed = NULL;
                     }
                 }
 
-                if(d->filename_speed && d->chart_var_speed) {
+                if (d->filename_speed && d->chart_var_speed) {
                     int ret = 0;
 
-                    if (d->carrier || !d->filename_carrier) {
+                    if ((d->carrier || d->carrier_file_exists) &&
+                        (d->speed_file_exists || now_monotonic_sec() - d->speed_file_lost_time > READ_RETRY_PERIOD)) {
                         ret = read_single_number_file(d->filename_speed, (unsigned long long *) &d->speed);
                     } else {
-                        d->speed = 0;
+                        d->speed = 0; // TODO: this is wrong, shouldn't use 0 value, but NULL.
                     }
 
                     if(ret) {
-                        error("Cannot refresh interface %s speed by reading '%s'. Will not update its speed anymore.", d->name, d->filename_speed);
-                        freez(d->filename_speed);
-                        d->filename_speed = NULL;
+                        if (d->speed_file_exists)
+                            collector_error("Cannot refresh interface %s speed by reading '%s'.", d->name, d->filename_speed);
+                        d->speed_file_exists = 0;
+                        d->speed_file_lost_time = now_monotonic_sec();
                     }
                     else {
-                        rrdsetvar_custom_chart_variable_set(d->chart_var_speed, (calculated_number) d->speed * KILOBITS_IN_A_MEGABIT);
-
                         if(d->do_speed != CONFIG_BOOLEAN_NO) {
                             if(unlikely(!d->st_speed)) {
                                 d->st_speed = rrdset_create_localhost(
@@ -947,22 +1040,27 @@ int do_proc_net_dev(int update_every, usec_t dt) {
 
                                 rrdset_flag_set(d->st_speed, RRDSET_FLAG_DETAIL);
 
-                                rrdset_update_labels(d->st_speed, d->chart_labels);
+                                rrdset_update_rrdlabels(d->st_speed, d->chart_labels);
 
                                 d->rd_speed = rrddim_add(d->st_speed, "speed",  NULL,  1, 1, RRD_ALGORITHM_ABSOLUTE);
                             }
-                            else rrdset_next(d->st_speed);
 
                             rrddim_set_by_pointer(d->st_speed, d->rd_speed, (collected_number)d->speed * KILOBITS_IN_A_MEGABIT);
                             rrdset_done(d->st_speed);
                         }
+
+                        rrdsetvar_custom_chart_variable_set(
+                            d->st_bandwidth, d->chart_var_speed, (NETDATA_DOUBLE)d->speed * KILOBITS_IN_A_MEGABIT);
+
+                        if (d->speed) {
+                            d->speed_file_exists = 1;
+                            d->speed_file_lost_time = 0;
+                        }
                     }
                 }
             }
         }
 
-        // --------------------------------------------------------------------
-
         if(d->do_duplex != CONFIG_BOOLEAN_NO && d->filename_duplex) {
             if(unlikely(!d->st_duplex)) {
                 d->st_duplex = rrdset_create_localhost(
@@ -982,18 +1080,19 @@ int do_proc_net_dev(int update_every, usec_t dt) {
 
                 rrdset_flag_set(d->st_duplex, RRDSET_FLAG_DETAIL);
 
-                rrdset_update_labels(d->st_duplex, d->chart_labels);
+                rrdset_update_rrdlabels(d->st_duplex, d->chart_labels);
 
-                d->rd_duplex = rrddim_add(d->st_duplex, "duplex",  NULL,  1, 1, RRD_ALGORITHM_ABSOLUTE);
+                d->rd_duplex_full = rrddim_add(d->st_duplex, "full", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
+                d->rd_duplex_half = rrddim_add(d->st_duplex, "half", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
+                d->rd_duplex_unknown = rrddim_add(d->st_duplex, "unknown", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
             }
-            else rrdset_next(d->st_duplex);
 
-            rrddim_set_by_pointer(d->st_duplex, d->rd_duplex, (collected_number)d->duplex);
+            rrddim_set_by_pointer(d->st_duplex, d->rd_duplex_full, (collected_number)(d->duplex == NETDEV_DUPLEX_FULL));
+            rrddim_set_by_pointer(d->st_duplex, d->rd_duplex_half, (collected_number)(d->duplex == NETDEV_DUPLEX_HALF));
+            rrddim_set_by_pointer(d->st_duplex, d->rd_duplex_unknown, (collected_number)(d->duplex == NETDEV_DUPLEX_UNKNOWN));
             rrdset_done(d->st_duplex);
         }
 
-        // --------------------------------------------------------------------
-
         if(d->do_operstate != CONFIG_BOOLEAN_NO && d->filename_operstate) {
             if(unlikely(!d->st_operstate)) {
                 d->st_operstate = rrdset_create_localhost(
@@ -1013,19 +1112,28 @@ int do_proc_net_dev(int update_every, usec_t dt) {
 
                 rrdset_flag_set(d->st_operstate, RRDSET_FLAG_DETAIL);
 
-                rrdset_update_labels(d->st_operstate, d->chart_labels);
+                rrdset_update_rrdlabels(d->st_operstate, d->chart_labels);
 
-                d->rd_operstate = rrddim_add(d->st_operstate, "state",  NULL,  1, 1, RRD_ALGORITHM_ABSOLUTE);
+                d->rd_operstate_up = rrddim_add(d->st_operstate, "up", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
+                d->rd_operstate_down = rrddim_add(d->st_operstate, "down", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
+                d->rd_operstate_notpresent = rrddim_add(d->st_operstate, "notpresent", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
+                d->rd_operstate_lowerlayerdown = rrddim_add(d->st_operstate, "lowerlayerdown", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
+                d->rd_operstate_testing = rrddim_add(d->st_operstate, "testing", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
+                d->rd_operstate_dormant = rrddim_add(d->st_operstate, "dormant", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
+                d->rd_operstate_unknown = rrddim_add(d->st_operstate, "unknown", NULL, 1, 1, RRD_ALGORITHM_ABSOLUTE);
             }
-            else rrdset_next(d->st_operstate);
 
-            rrddim_set_by_pointer(d->st_operstate, d->rd_operstate, (collected_number)d->operstate);
+            rrddim_set_by_pointer(d->st_operstate, d->rd_operstate_up, (collected_number)(d->operstate == NETDEV_OPERSTATE_UP));
+            rrddim_set_by_pointer(d->st_operstate, d->rd_operstate_down, (collected_number)(d->operstate == NETDEV_OPERSTATE_DOWN));
+            rrddim_set_by_pointer(d->st_operstate, d->rd_operstate_notpresent, (collected_number)(d->operstate == NETDEV_OPERSTATE_NOTPRESENT));
+            rrddim_set_by_pointer(d->st_operstate, d->rd_operstate_lowerlayerdown, (collected_number)(d->operstate == NETDEV_OPERSTATE_LOWERLAYERDOWN));
+            rrddim_set_by_pointer(d->st_operstate, d->rd_operstate_testing, (collected_number)(d->operstate == NETDEV_OPERSTATE_TESTING));
+            rrddim_set_by_pointer(d->st_operstate, d->rd_operstate_dormant, (collected_number)(d->operstate == NETDEV_OPERSTATE_DORMANT));
+            rrddim_set_by_pointer(d->st_operstate, d->rd_operstate_unknown, (collected_number)(d->operstate == NETDEV_OPERSTATE_UNKNOWN));
             rrdset_done(d->st_operstate);
         }
 
-        // --------------------------------------------------------------------
-
-        if(d->do_carrier != CONFIG_BOOLEAN_NO && d->filename_carrier) {
+        if(d->do_carrier != CONFIG_BOOLEAN_NO && d->carrier_file_exists) {
             if(unlikely(!d->st_carrier)) {
                 d->st_carrier = rrdset_create_localhost(
                         d->chart_type_net_carrier
@@ -1044,18 +1152,17 @@ int do_proc_net_dev(int update_every, usec_t dt) {
 
                 rrdset_flag_set(d->st_carrier, RRDSET_FLAG_DETAIL);
 
-                rrdset_update_labels(d->st_carrier, d->chart_labels);
+                rrdset_update_rrdlabels(d->st_carrier, d->chart_labels);
 
-                d->rd_carrier = rrddim_add(d->st_carrier, "carrier",  NULL,  1, 1, RRD_ALGORITHM_ABSOLUTE);
+                d->rd_carrier_up = rrddim_add(d->st_carrier, "up",  NULL,  1, 1, RRD_ALGORITHM_ABSOLUTE);
+                d->rd_carrier_down = rrddim_add(d->st_carrier, "down",  NULL,  1, 1, RRD_ALGORITHM_ABSOLUTE);
             }
-            else rrdset_next(d->st_carrier);
 
-            rrddim_set_by_pointer(d->st_carrier, d->rd_carrier, (collected_number)d->carrier);
+            rrddim_set_by_pointer(d->st_carrier, d->rd_carrier_up, (collected_number)(d->carrier == 1));
+            rrddim_set_by_pointer(d->st_carrier, d->rd_carrier_down, (collected_number)(d->carrier != 1));
             rrdset_done(d->st_carrier);
         }
 
-        // --------------------------------------------------------------------
-
         if(d->do_mtu != CONFIG_BOOLEAN_NO && d->filename_mtu) {
             if(unlikely(!d->st_mtu)) {
                 d->st_mtu = rrdset_create_localhost(
@@ -1075,18 +1182,15 @@ int do_proc_net_dev(int update_every, usec_t dt) {
 
                 rrdset_flag_set(d->st_mtu, RRDSET_FLAG_DETAIL);
 
-                rrdset_update_labels(d->st_mtu, d->chart_labels);
+                rrdset_update_rrdlabels(d->st_mtu, d->chart_labels);
 
                 d->rd_mtu = rrddim_add(d->st_mtu, "mtu",  NULL,  1, 1, RRD_ALGORITHM_ABSOLUTE);
             }
-            else rrdset_next(d->st_mtu);
 
             rrddim_set_by_pointer(d->st_mtu, d->rd_mtu, (collected_number)d->mtu);
             rrdset_done(d->st_mtu);
         }
 
-        // --------------------------------------------------------------------
-
         if(unlikely(d->do_packets == CONFIG_BOOLEAN_AUTO &&
            (d->rpackets || d->tpackets || d->rmulticast || netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES)))
             d->do_packets = CONFIG_BOOLEAN_YES;
@@ -1111,7 +1215,7 @@ int do_proc_net_dev(int update_every, usec_t dt) {
 
                 rrdset_flag_set(d->st_packets, RRDSET_FLAG_DETAIL);
 
-                rrdset_update_labels(d->st_packets, d->chart_labels);
+                rrdset_update_rrdlabels(d->st_packets, d->chart_labels);
 
                 d->rd_rpackets   = rrddim_add(d->st_packets, "received",  NULL,  1, 1, RRD_ALGORITHM_INCREMENTAL);
                 d->rd_tpackets   = rrddim_add(d->st_packets, "sent",      NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -1125,7 +1229,6 @@ int do_proc_net_dev(int update_every, usec_t dt) {
                     d->rd_tpackets = td;
                 }
             }
-            else rrdset_next(d->st_packets);
 
             rrddim_set_by_pointer(d->st_packets, d->rd_rpackets, (collected_number)d->rpackets);
             rrddim_set_by_pointer(d->st_packets, d->rd_tpackets, (collected_number)d->tpackets);
@@ -1133,8 +1236,6 @@ int do_proc_net_dev(int update_every, usec_t dt) {
             rrdset_done(d->st_packets);
         }
 
-        // --------------------------------------------------------------------
-
         if(unlikely(d->do_errors == CONFIG_BOOLEAN_AUTO &&
                     (d->rerrors || d->terrors || netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES)))
             d->do_errors = CONFIG_BOOLEAN_YES;
@@ -1159,7 +1260,7 @@ int do_proc_net_dev(int update_every, usec_t dt) {
 
                 rrdset_flag_set(d->st_errors, RRDSET_FLAG_DETAIL);
 
-                rrdset_update_labels(d->st_errors, d->chart_labels);
+                rrdset_update_rrdlabels(d->st_errors, d->chart_labels);
 
                 d->rd_rerrors = rrddim_add(d->st_errors, "inbound",  NULL,  1, 1, RRD_ALGORITHM_INCREMENTAL);
                 d->rd_terrors = rrddim_add(d->st_errors, "outbound", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -1172,15 +1273,12 @@ int do_proc_net_dev(int update_every, usec_t dt) {
                     d->rd_terrors = td;
                 }
             }
-            else rrdset_next(d->st_errors);
 
             rrddim_set_by_pointer(d->st_errors, d->rd_rerrors, (collected_number)d->rerrors);
             rrddim_set_by_pointer(d->st_errors, d->rd_terrors, (collected_number)d->terrors);
             rrdset_done(d->st_errors);
         }
 
-        // --------------------------------------------------------------------
-
         if(unlikely(d->do_drops == CONFIG_BOOLEAN_AUTO &&
                     (d->rdrops || d->tdrops || netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES)))
             d->do_drops = CONFIG_BOOLEAN_YES;
@@ -1205,7 +1303,7 @@ int do_proc_net_dev(int update_every, usec_t dt) {
 
                 rrdset_flag_set(d->st_drops, RRDSET_FLAG_DETAIL);
 
-                rrdset_update_labels(d->st_drops, d->chart_labels);
+                rrdset_update_rrdlabels(d->st_drops, d->chart_labels);
 
                 d->rd_rdrops = rrddim_add(d->st_drops, "inbound",  NULL,  1, 1, RRD_ALGORITHM_INCREMENTAL);
                 d->rd_tdrops = rrddim_add(d->st_drops, "outbound", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -1218,15 +1316,12 @@ int do_proc_net_dev(int update_every, usec_t dt) {
                     d->rd_tdrops = td;
                 }
             }
-            else rrdset_next(d->st_drops);
 
             rrddim_set_by_pointer(d->st_drops, d->rd_rdrops, (collected_number)d->rdrops);
             rrddim_set_by_pointer(d->st_drops, d->rd_tdrops, (collected_number)d->tdrops);
             rrdset_done(d->st_drops);
         }
 
-        // --------------------------------------------------------------------
-
         if(unlikely(d->do_fifo == CONFIG_BOOLEAN_AUTO &&
                     (d->rfifo || d->tfifo || netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES)))
             d->do_fifo = CONFIG_BOOLEAN_YES;
@@ -1251,7 +1346,7 @@ int do_proc_net_dev(int update_every, usec_t dt) {
 
                 rrdset_flag_set(d->st_fifo, RRDSET_FLAG_DETAIL);
 
-                rrdset_update_labels(d->st_fifo, d->chart_labels);
+                rrdset_update_rrdlabels(d->st_fifo, d->chart_labels);
 
                 d->rd_rfifo = rrddim_add(d->st_fifo, "receive",  NULL,  1, 1, RRD_ALGORITHM_INCREMENTAL);
                 d->rd_tfifo = rrddim_add(d->st_fifo, "transmit", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -1264,15 +1359,12 @@ int do_proc_net_dev(int update_every, usec_t dt) {
                     d->rd_tfifo = td;
                 }
             }
-            else rrdset_next(d->st_fifo);
 
             rrddim_set_by_pointer(d->st_fifo, d->rd_rfifo, (collected_number)d->rfifo);
             rrddim_set_by_pointer(d->st_fifo, d->rd_tfifo, (collected_number)d->tfifo);
             rrdset_done(d->st_fifo);
         }
 
-        // --------------------------------------------------------------------
-
         if(unlikely(d->do_compressed == CONFIG_BOOLEAN_AUTO &&
                     (d->rcompressed || d->tcompressed || netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES)))
             d->do_compressed = CONFIG_BOOLEAN_YES;
@@ -1297,7 +1389,7 @@ int do_proc_net_dev(int update_every, usec_t dt) {
 
                 rrdset_flag_set(d->st_compressed, RRDSET_FLAG_DETAIL);
 
-                rrdset_update_labels(d->st_compressed, d->chart_labels);
+                rrdset_update_rrdlabels(d->st_compressed, d->chart_labels);
 
                 d->rd_rcompressed = rrddim_add(d->st_compressed, "received", NULL,  1, 1, RRD_ALGORITHM_INCREMENTAL);
                 d->rd_tcompressed = rrddim_add(d->st_compressed, "sent",     NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
@@ -1310,15 +1402,12 @@ int do_proc_net_dev(int update_every, usec_t dt) {
                     d->rd_tcompressed = td;
                 }
             }
-            else rrdset_next(d->st_compressed);
 
             rrddim_set_by_pointer(d->st_compressed, d->rd_rcompressed, (collected_number)d->rcompressed);
             rrddim_set_by_pointer(d->st_compressed, d->rd_tcompressed, (collected_number)d->tcompressed);
             rrdset_done(d->st_compressed);
         }
 
-        // --------------------------------------------------------------------
-
         if(unlikely(d->do_events == CONFIG_BOOLEAN_AUTO &&
                     (d->rframe || d->tcollisions || d->tcarrier || netdata_zero_metrics_enabled == CONFIG_BOOLEAN_YES)))
             d->do_events = CONFIG_BOOLEAN_YES;
@@ -1343,13 +1432,12 @@ int do_proc_net_dev(int update_every, usec_t dt) {
 
                 rrdset_flag_set(d->st_events, RRDSET_FLAG_DETAIL);
 
-                rrdset_update_labels(d->st_events, d->chart_labels);
+                rrdset_update_rrdlabels(d->st_events, d->chart_labels);
 
                 d->rd_rframe      = rrddim_add(d->st_events, "frames",     NULL,  1, 1, RRD_ALGORITHM_INCREMENTAL);
                 d->rd_tcollisions = rrddim_add(d->st_events, "collisions", NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
                 d->rd_tcarrier    = rrddim_add(d->st_events, "carrier",    NULL, -1, 1, RRD_ALGORITHM_INCREMENTAL);
             }
-            else rrdset_next(d->st_events);
 
             rrddim_set_by_pointer(d->st_events, d->rd_rframe,      (collected_number)d->rframe);
             rrddim_set_by_pointer(d->st_events, d->rd_tcollisions, (collected_number)d->tcollisions);
@@ -1384,8 +1472,6 @@ int do_proc_net_dev(int update_every, usec_t dt) {
             rd_in  = rrddim_add(st_system_net, "InOctets",  "received", 8, BITS_IN_A_KILOBIT, RRD_ALGORITHM_INCREMENTAL);
             rd_out = rrddim_add(st_system_net, "OutOctets", "sent",    -8, BITS_IN_A_KILOBIT, RRD_ALGORITHM_INCREMENTAL);
         }
-        else
-            rrdset_next(st_system_net);
 
         rrddim_set_by_pointer(st_system_net, rd_in,  (collected_number)system_rbytes);
         rrddim_set_by_pointer(st_system_net, rd_out, (collected_number)system_tbytes);
@@ -1402,7 +1488,7 @@ static void netdev_main_cleanup(void *ptr)
 {
     UNUSED(ptr);
 
-    info("cleaning up...");
+    collector_info("cleaning up...");
 
     worker_unregister();
 }
@@ -1418,11 +1504,11 @@ void *netdev_main(void *ptr)
     heartbeat_t hb;
     heartbeat_init(&hb);
 
-    while (!netdata_exit) {
+    while (service_running(SERVICE_COLLECTORS)) {
         worker_is_idle();
         usec_t hb_dt = heartbeat_next(&hb, step);
 
-        if (unlikely(netdata_exit))
+        if (unlikely(!service_running(SERVICE_COLLECTORS)))
             break;
 
         worker_is_busy(0);
