@@ -4,6 +4,7 @@
 #define NETDATA_COMMON_H 1
 
 #include "libnetdata/libnetdata.h"
+#include "event_loop.h"
 
 // ----------------------------------------------------------------------------
 // shortcuts for the default netdata configuration
@@ -69,7 +70,7 @@
 #include "claim/claim.h"
 
 // netdata agent cloud link
-#include "aclk/aclk_api.h"
+#include "aclk/aclk.h"
 
 // global GUID map functions
 
@@ -84,9 +85,6 @@
 #include "commands.h"
 #include "analytics.h"
 
-// metric correlations
-#include "database/metric_correlations.h"
-
 // global netdata daemon variables
 extern char *netdata_configured_hostname;
 extern char *netdata_configured_user_config_dir;
@@ -108,4 +106,6 @@ extern int netdata_anonymous_statistics_enabled;
 extern int netdata_ready;
 extern int netdata_cloud_setting;
 
+long get_netdata_cpus(void);
+
 #endif /* NETDATA_COMMON_H */
