@@ -64,7 +64,7 @@ int mark_scheduled_instances(struct engine *engine)
  * @param last_timestamp the timestamp that should be reported to the exporting connector instance.
  * @return Returns the value, calculated over the given period.
  */
-calculated_number exporting_calculate_value_from_stored_data(
+NETDATA_DOUBLE exporting_calculate_value_from_stored_data(
     struct instance *instance,
     RRDDIM *rd,
     time_t *last_timestamp)
@@ -77,11 +77,10 @@ calculated_number exporting_calculate_value_from_stored_data(
     time_t before = instance->before;
 
     // find the edges of the rrd database for this chart
-    time_t first_t = rd->state->query_ops.oldest_time(rd);
-    time_t last_t = rd->state->query_ops.latest_time(rd);
+    time_t first_t = rd->tiers[0].query_ops->oldest_time_s(rd->tiers[0].db_metric_handle);
+    time_t last_t = rd->tiers[0].query_ops->latest_time_s(rd->tiers[0].db_metric_handle);
     time_t update_every = st->update_every;
-    struct rrddim_query_handle handle;
-    storage_number n;
+    struct storage_engine_query_handle handle;
 
     // step back a little, to make sure we have complete data collection
     // for all metrics
@@ -111,9 +110,9 @@ calculated_number exporting_calculate_value_from_stored_data(
         debug(
             D_EXPORTING,
             "EXPORTING: %s.%s.%s: aligned timeframe %lu to %lu is outside the chart's database range %lu to %lu",
-            host->hostname,
-            st->id,
-            rd->id,
+            rrdhost_hostname(host),
+            rrdset_id(st),
+            rrddim_id(rd),
             (unsigned long)after,
             (unsigned long)before,
             (unsigned long)first_t,
@@ -123,31 +122,32 @@ calculated_number exporting_calculate_value_from_stored_data(
 
     *last_timestamp = before;
 
+    size_t points_read = 0;
     size_t counter = 0;
-    calculated_number sum = 0;
+    NETDATA_DOUBLE sum = 0;
 
-    for (rd->state->query_ops.init(rd, &handle, after, before); !rd->state->query_ops.is_finished(&handle);) {
-        time_t curr_t;
-        n = rd->state->query_ops.next_metric(&handle, &curr_t);
+    for (rd->tiers[0].query_ops->init(rd->tiers[0].db_metric_handle, &handle, after, before, STORAGE_PRIORITY_LOW); !rd->tiers[0].query_ops->is_finished(&handle);) {
+        STORAGE_POINT sp = rd->tiers[0].query_ops->next_metric(&handle);
+        points_read++;
 
-        if (unlikely(!does_storage_number_exist(n))) {
+        if (unlikely(storage_point_is_gap(sp))) {
             // not collected
             continue;
         }
 
-        calculated_number value = unpack_storage_number(n);
-        sum += value;
-
-        counter++;
+        sum += sp.sum;
+        counter += sp.count;
     }
-    rd->state->query_ops.finalize(&handle);
+    rd->tiers[0].query_ops->finalize(&handle);
+    global_statistics_exporters_query_completed(points_read);
+
     if (unlikely(!counter)) {
         debug(
             D_EXPORTING,
             "EXPORTING: %s.%s.%s: no values stored in database for range %lu to %lu",
-            host->hostname,
-            st->id,
-            rd->id,
+            rrdhost_hostname(host),
+            rrdset_id(st),
+            rrddim_id(rd),
             (unsigned long)after,
             (unsigned long)before);
         return NAN;
@@ -156,7 +156,7 @@ calculated_number exporting_calculate_value_from_stored_data(
     if (unlikely(EXPORTING_OPTIONS_DATA_SOURCE(instance->config.options) == EXPORTING_SOURCE_DATA_SUM))
         return sum;
 
-    return sum / (calculated_number)counter;
+    return sum / (NETDATA_DOUBLE)counter;
 }
 
 /**
@@ -261,6 +261,27 @@ void end_chart_formatting(struct engine *engine, RRDSET *st)
     }
 }
 
+/**
+ * Format variables for every connector instance's buffer
+ *
+ * @param engine an engine data structure.
+ * @param host a data collecting host.
+  */
+void variables_formatting(struct engine *engine, RRDHOST *host)
+{
+    for (struct instance *instance = engine->instance_root; instance; instance = instance->next) {
+        if (instance->scheduled && !instance->skip_host && should_send_variables(instance)) {
+            if (instance->variables_formatting && instance->variables_formatting(instance, host) != 0){ 
+                error("EXPORTING: cannot format variables for %s", instance->config.name);
+                disable_instance(instance);
+                continue;
+            }
+            // sum all variables as one metrics
+            instance->stats.buffered_metrics++;
+        }
+    }
+}
+
 /**
  * End host formatting for every connector instance's buffer
  *
@@ -320,26 +341,22 @@ void prepare_buffers(struct engine *engine)
 
     rrd_rdlock();
     RRDHOST *host;
-    rrdhost_foreach_read(host)
-    {
-        rrdhost_rdlock(host);
+    rrdhost_foreach_read(host) {
         start_host_formatting(engine, host);
         RRDSET *st;
-        rrdset_foreach_read(st, host)
-        {
-            rrdset_rdlock(st);
+        rrdset_foreach_read(st, host) {
             start_chart_formatting(engine, st);
 
             RRDDIM *rd;
             rrddim_foreach_read(rd, st)
                 metric_formatting(engine, rd);
+            rrddim_foreach_done(rd);
 
             end_chart_formatting(engine, st);
-            rrdset_unlock(st);
         }
-
+        rrdset_foreach_done(st);
+        variables_formatting(engine, host);
         end_host_formatting(engine, host);
-        rrdhost_unlock(host);
     }
     rrd_unlock();
     netdata_thread_enable_cancelability();
@@ -358,8 +375,8 @@ int flush_host_labels(struct instance *instance, RRDHOST *host)
 {
     (void)host;
 
-    if (instance->labels)
-        buffer_flush(instance->labels);
+    if (instance->labels_buffer)
+        buffer_flush(instance->labels_buffer);
 
     return 0;
 }
@@ -380,7 +397,7 @@ int simple_connector_end_batch(struct instance *instance)
     struct simple_connector_buffer *last_buffer = simple_connector_data->last_buffer;
 
     if (!last_buffer->buffer) {
-        last_buffer->buffer = buffer_create(0);
+        last_buffer->buffer = buffer_create(0, &netdata_buffers_statistics.buffers_exporters);
     }
 
     if (last_buffer->used) {
@@ -402,7 +419,7 @@ int simple_connector_end_batch(struct instance *instance)
     if (last_buffer->header)
         buffer_flush(last_buffer->header);
     else
-        last_buffer->header = buffer_create(0);
+        last_buffer->header = buffer_create(0, &netdata_buffers_statistics.buffers_exporters);
 
     if (instance->prepare_header)
         instance->prepare_header(instance);
