@@ -9,9 +9,9 @@
 
 static int is_matches_rrdset(struct instance *instance, RRDSET *st, SIMPLE_PATTERN *filter) {
     if (instance->config.options & EXPORTING_OPTION_SEND_NAMES) {
-        return simple_pattern_matches(filter, st->name);
+        return simple_pattern_matches(filter, rrdset_name(st));
     }
-    return simple_pattern_matches(filter, st->id);
+    return simple_pattern_matches(filter, rrdset_id(st));
 }
 
 /**
@@ -28,10 +28,6 @@ inline int can_send_rrdset(struct instance *instance, RRDSET *st, SIMPLE_PATTERN
     RRDHOST *host = st->rrdhost;
 #endif
 
-    // Do not send anomaly rates charts.
-    if (st->state && st->state->is_ar_chart)
-        return 0;
-
     if (unlikely(rrdset_flag_check(st, RRDSET_FLAG_EXPORTING_IGNORE)))
         return 0;
 
@@ -48,8 +44,8 @@ inline int can_send_rrdset(struct instance *instance, RRDSET *st, SIMPLE_PATTERN
             debug(
                 D_EXPORTING,
                 "EXPORTING: not sending chart '%s' of host '%s', because it is disabled for exporting.",
-                st->id,
-                host->hostname);
+                rrdset_id(st),
+                rrdhost_hostname(host));
             return 0;
         }
     }
@@ -58,8 +54,8 @@ inline int can_send_rrdset(struct instance *instance, RRDSET *st, SIMPLE_PATTERN
         debug(
             D_EXPORTING,
             "EXPORTING: not sending chart '%s' of host '%s', because it is not available for exporting.",
-            st->id,
-            host->hostname);
+            rrdset_id(st),
+            rrdhost_hostname(host));
         return 0;
     }
 
@@ -69,8 +65,8 @@ inline int can_send_rrdset(struct instance *instance, RRDSET *st, SIMPLE_PATTERN
         debug(
             D_EXPORTING,
             "EXPORTING: not sending chart '%s' of host '%s' because its memory mode is '%s' and the exporting connector requires database access.",
-            st->id,
-            host->hostname,
+            rrdset_id(st),
+            rrdhost_hostname(host),
             rrd_memory_mode_name(host->rrd_memory_mode));
         return 0;
     }
@@ -290,35 +286,44 @@ inline char *prometheus_units_copy(char *d, const char *s, size_t usable, int sh
  * @param instance an instance data structure.
  * @param host a data collecting host.
  */
+
+struct format_prometheus_label_callback {
+    struct instance *instance;
+    size_t count;
+};
+
+static int format_prometheus_label_callback(const char *name, const char *value, RRDLABEL_SRC ls, void *data) {
+    struct format_prometheus_label_callback *d = (struct format_prometheus_label_callback *)data;
+
+    if (!should_send_label(d->instance, ls)) return 0;
+
+    char k[PROMETHEUS_ELEMENT_MAX + 1];
+    char v[PROMETHEUS_ELEMENT_MAX + 1];
+
+    prometheus_name_copy(k, name, PROMETHEUS_ELEMENT_MAX);
+    prometheus_label_copy(v, value, PROMETHEUS_ELEMENT_MAX);
+
+    if (*k && *v) {
+        if (d->count > 0) buffer_strcat(d->instance->labels_buffer, ",");
+        buffer_sprintf(d->instance->labels_buffer, "%s=\"%s\"", k, v);
+        d->count++;
+    }
+    return 1;
+}
+
 void format_host_labels_prometheus(struct instance *instance, RRDHOST *host)
 {
     if (unlikely(!sending_labels_configured(instance)))
         return;
 
-    if (!instance->labels)
-        instance->labels = buffer_create(1024);
+    if (!instance->labels_buffer)
+        instance->labels_buffer = buffer_create(1024, &netdata_buffers_statistics.buffers_exporters);
 
-    int count = 0;
-    rrdhost_check_rdlock(host);
-    netdata_rwlock_rdlock(&host->labels.labels_rwlock);
-    for (struct label *label = host->labels.head; label; label = label->next) {
-        if (!should_send_label(instance, label))
-            continue;
-
-        char key[PROMETHEUS_ELEMENT_MAX + 1];
-        char value[PROMETHEUS_ELEMENT_MAX + 1];
-
-        prometheus_name_copy(key, label->key, PROMETHEUS_ELEMENT_MAX);
-        prometheus_label_copy(value, label->value, PROMETHEUS_ELEMENT_MAX);
-
-        if (*key && *value) {
-            if (count > 0)
-                buffer_strcat(instance->labels, ",");
-            buffer_sprintf(instance->labels, "%s=\"%s\"", key, value);
-            count++;
-        }
-    }
-    netdata_rwlock_unlock(&host->labels.labels_rwlock);
+    struct format_prometheus_label_callback tmp = {
+        .instance = instance,
+        .count = 0
+    };
+    rrdlabels_walkthrough_read(host->rrdlabels, format_prometheus_label_callback, &tmp);
 }
 
 struct host_variables_callback_options {
@@ -340,11 +345,12 @@ struct host_variables_callback_options {
  * @param data callback options.
  * @return Returns 1 if the chart can be sent, 0 otherwise.
  */
-static int print_host_variables(RRDVAR *rv, void *data)
-{
+static int print_host_variables_callback(const DICTIONARY_ITEM *item __maybe_unused, void *rv_ptr __maybe_unused, void *data) {
+    const RRDVAR_ACQUIRED *rv = (const RRDVAR_ACQUIRED *)item;
+
     struct host_variables_callback_options *opts = data;
 
-    if (rv->options & (RRDVAR_OPTION_CUSTOM_HOST_VAR | RRDVAR_OPTION_CUSTOM_CHART_VAR)) {
+    if (rrdvar_flags(rv) & (RRDVAR_FLAG_CUSTOM_HOST_VAR | RRDVAR_FLAG_CUSTOM_CHART_VAR)) {
         if (!opts->host_header_printed) {
             opts->host_header_printed = 1;
 
@@ -353,11 +359,11 @@ static int print_host_variables(RRDVAR *rv, void *data)
             }
         }
 
-        calculated_number value = rrdvar2number(rv);
+        NETDATA_DOUBLE value = rrdvar2number(rv);
         if (isnan(value) || isinf(value)) {
             if (opts->output_options & PROMETHEUS_OUTPUT_HELP)
                 buffer_sprintf(
-                    opts->wb, "# COMMENT variable \"%s\" is %s. Skipped.\n", rv->name, (isnan(value)) ? "NAN" : "INF");
+                    opts->wb, "# COMMENT variable \"%s\" is %s. Skipped.\n", rrdvar_name(rv), (isnan(value)) ? "NAN" : "INF");
 
             return 0;
         }
@@ -369,12 +375,12 @@ static int print_host_variables(RRDVAR *rv, void *data)
             label_post = "}";
         }
 
-        prometheus_name_copy(opts->name, rv->name, sizeof(opts->name));
+        prometheus_name_copy(opts->name, rrdvar_name(rv), sizeof(opts->name));
 
         if (opts->output_options & PROMETHEUS_OUTPUT_TIMESTAMPS)
             buffer_sprintf(
                 opts->wb,
-                "%s_%s%s%s%s " CALCULATED_NUMBER_FORMAT " %llu\n",
+                "%s_%s%s%s%s " NETDATA_DOUBLE_FORMAT " %llu\n",
                 opts->prefix,
                 opts->name,
                 label_pre,
@@ -385,7 +391,7 @@ static int print_host_variables(RRDVAR *rv, void *data)
         else
             buffer_sprintf(
                 opts->wb,
-                "%s_%s%s%s%s " CALCULATED_NUMBER_FORMAT "\n",
+                "%s_%s%s%s%s " NETDATA_DOUBLE_FORMAT "\n",
                 opts->prefix,
                 opts->name,
                 label_pre,
@@ -436,17 +442,17 @@ static void generate_as_collected_prom_help(BUFFER *wb, struct gen_parameters *p
         wb,
         "%s: chart \"%s\", context \"%s\", family \"%s\", dimension \"%s\", value * ",
         p->suffix,
-        (p->output_options & PROMETHEUS_OUTPUT_NAMES && p->st->name) ? p->st->name : p->st->id,
-        p->st->context,
-        p->st->family,
-        (p->output_options & PROMETHEUS_OUTPUT_NAMES && p->rd->name) ? p->rd->name : p->rd->id);
+        (p->output_options & PROMETHEUS_OUTPUT_NAMES && p->st->name) ? rrdset_name(p->st) : rrdset_id(p->st),
+        rrdset_context(p->st),
+        rrdset_family(p->st),
+        (p->output_options & PROMETHEUS_OUTPUT_NAMES && p->rd->name) ? rrddim_name(p->rd) : rrddim_id(p->rd));
 
     if (prometheus_collector)
         buffer_sprintf(wb, "1 / 1");
     else
         buffer_sprintf(wb, COLLECTED_NUMBER_FORMAT " / " COLLECTED_NUMBER_FORMAT, p->rd->multiplier, p->rd->divisor);
 
-    buffer_sprintf(wb, " %s %s (%s)\n", p->relation, p->st->units, p->type);
+    buffer_sprintf(wb, " %s %s (%s)\n", p->relation, rrdset_units(p->st), p->type);
 }
 
 /**
@@ -474,9 +480,9 @@ static void generate_as_collected_prom_metric(BUFFER *wb, struct gen_parameters
     if (prometheus_collector)
         buffer_sprintf(
             wb,
-            CALCULATED_NUMBER_FORMAT,
-            (calculated_number)p->rd->last_collected_value * (calculated_number)p->rd->multiplier /
-                (calculated_number)p->rd->divisor);
+            NETDATA_DOUBLE_FORMAT,
+            (NETDATA_DOUBLE)p->rd->last_collected_value * (NETDATA_DOUBLE)p->rd->multiplier /
+                (NETDATA_DOUBLE)p->rd->divisor);
     else
         buffer_sprintf(wb, COLLECTED_NUMBER_FORMAT, p->rd->last_collected_value);
 
@@ -509,115 +515,66 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(
     PROMETHEUS_OUTPUT_OPTIONS output_options)
 {
     SIMPLE_PATTERN *filter = simple_pattern_create(filter_string, NULL, SIMPLE_PATTERN_EXACT);
-    rrdhost_rdlock(host);
 
     char hostname[PROMETHEUS_ELEMENT_MAX + 1];
-    prometheus_label_copy(hostname, host->hostname, PROMETHEUS_ELEMENT_MAX);
+    prometheus_label_copy(hostname, rrdhost_hostname(host), PROMETHEUS_ELEMENT_MAX);
 
     format_host_labels_prometheus(instance, host);
 
+    buffer_sprintf(
+        wb,
+        "netdata_info{instance=\"%s\",application=\"%s\",version=\"%s\"",
+        hostname,
+        rrdhost_program_name(host),
+        rrdhost_program_version(host));
+
+    if (instance->labels_buffer && *buffer_tostring(instance->labels_buffer)) {
+        buffer_sprintf(wb, ",%s", buffer_tostring(instance->labels_buffer));
+    }
+
     if (output_options & PROMETHEUS_OUTPUT_TIMESTAMPS)
-        buffer_sprintf(
-            wb,
-            "netdata_info{instance=\"%s\",application=\"%s\",version=\"%s\"} 1 %llu\n",
-            hostname,
-            host->program_name,
-            host->program_version,
-            now_realtime_usec() / USEC_PER_MS);
+        buffer_sprintf(wb, "} 1 %llu\n", now_realtime_usec() / USEC_PER_MS);
     else
-        buffer_sprintf(
-            wb,
-            "netdata_info{instance=\"%s\",application=\"%s\",version=\"%s\"} 1\n",
-            hostname,
-            host->program_name,
-            host->program_version);
+        buffer_sprintf(wb, "} 1\n");
 
     char labels[PROMETHEUS_LABELS_MAX + 1] = "";
     if (allhosts) {
-        if (instance->labels && buffer_tostring(instance->labels)) {
-            if (output_options & PROMETHEUS_OUTPUT_TIMESTAMPS) {
-                buffer_sprintf(
-                    wb,
-                    "netdata_host_tags_info{instance=\"%s\",%s} 1 %llu\n",
-                    hostname,
-                    buffer_tostring(instance->labels),
-                    now_realtime_usec() / USEC_PER_MS);
-
-                // deprecated, exists only for compatibility with older queries
-                buffer_sprintf(
-                    wb,
-                    "netdata_host_tags{instance=\"%s\",%s} 1 %llu\n",
-                    hostname,
-                    buffer_tostring(instance->labels),
-                    now_realtime_usec() / USEC_PER_MS);
-            } else {
-                buffer_sprintf(
-                    wb, "netdata_host_tags_info{instance=\"%s\",%s} 1\n", hostname, buffer_tostring(instance->labels));
-
-                // deprecated, exists only for compatibility with older queries
-                buffer_sprintf(
-                    wb, "netdata_host_tags{instance=\"%s\",%s} 1\n", hostname, buffer_tostring(instance->labels));
-            }
-        }
-
         snprintfz(labels, PROMETHEUS_LABELS_MAX, ",instance=\"%s\"", hostname);
-    } else {
-        if (instance->labels && buffer_tostring(instance->labels)) {
-            if (output_options & PROMETHEUS_OUTPUT_TIMESTAMPS) {
-                buffer_sprintf(
-                    wb,
-                    "netdata_host_tags_info{%s} 1 %llu\n",
-                    buffer_tostring(instance->labels),
-                    now_realtime_usec() / USEC_PER_MS);
+     }
 
-                // deprecated, exists only for compatibility with older queries
-                buffer_sprintf(
-                    wb,
-                    "netdata_host_tags{%s} 1 %llu\n",
-                    buffer_tostring(instance->labels),
-                    now_realtime_usec() / USEC_PER_MS);
-            } else {
-                buffer_sprintf(wb, "netdata_host_tags_info{%s} 1\n", buffer_tostring(instance->labels));
-
-                // deprecated, exists only for compatibility with older queries
-                buffer_sprintf(wb, "netdata_host_tags{%s} 1\n", buffer_tostring(instance->labels));
-            }
-        }
-    }
-
-    if (instance->labels)
-        buffer_flush(instance->labels);
+    if (instance->labels_buffer)
+        buffer_flush(instance->labels_buffer);
 
     // send custom variables set for the host
     if (output_options & PROMETHEUS_OUTPUT_VARIABLES) {
-        struct host_variables_callback_options opts = { .host = host,
-                                                        .wb = wb,
-                                                        .labels = (labels[0] == ',') ? &labels[1] : labels,
-                                                        .exporting_options = exporting_options,
-                                                        .output_options = output_options,
-                                                        .prefix = prefix,
-                                                        .now = now_realtime_sec(),
-                                                        .host_header_printed = 0 };
-        foreach_host_variable_callback(host, print_host_variables, &opts);
+
+        struct host_variables_callback_options opts = {
+            .host = host,
+            .wb = wb,
+            .labels = (labels[0] == ',') ? &labels[1] : labels,
+            .exporting_options = exporting_options,
+            .output_options = output_options,
+            .prefix = prefix,
+            .now = now_realtime_sec(),
+            .host_header_printed = 0
+        };
+
+        rrdvar_walkthrough_read(host->rrdvars, print_host_variables_callback, &opts);
     }
 
     // for each chart
     RRDSET *st;
-    rrdset_foreach_read(st, host)
-    {
+    rrdset_foreach_read(st, host) {
 
         if (likely(can_send_rrdset(instance, st, filter))) {
-            rrdset_rdlock(st);
-
             char chart[PROMETHEUS_ELEMENT_MAX + 1];
             char context[PROMETHEUS_ELEMENT_MAX + 1];
             char family[PROMETHEUS_ELEMENT_MAX + 1];
             char units[PROMETHEUS_ELEMENT_MAX + 1] = "";
 
-            prometheus_label_copy(
-                chart, (output_options & PROMETHEUS_OUTPUT_NAMES && st->name) ? st->name : st->id, PROMETHEUS_ELEMENT_MAX);
-            prometheus_label_copy(family, st->family, PROMETHEUS_ELEMENT_MAX);
-            prometheus_name_copy(context, st->context, PROMETHEUS_ELEMENT_MAX);
+            prometheus_label_copy(chart, (output_options & PROMETHEUS_OUTPUT_NAMES && st->name) ? rrdset_name(st) : rrdset_id(st), PROMETHEUS_ELEMENT_MAX);
+            prometheus_label_copy(family, rrdset_family(st), PROMETHEUS_ELEMENT_MAX);
+            prometheus_name_copy(context, rrdset_context(st), PROMETHEUS_ELEMENT_MAX);
 
             int as_collected = (EXPORTING_OPTIONS_DATA_SOURCE(exporting_options) == EXPORTING_SOURCE_DATA_AS_COLLECTED);
             int homogeneous = 1;
@@ -629,13 +586,13 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(
                 if (rrdset_flag_check(st, RRDSET_FLAG_HETEROGENEOUS))
                     homogeneous = 0;
 
-                if (st->module_name && !strcmp(st->module_name, "prometheus"))
+                if (!strcmp(rrdset_module_name(st), "prometheus"))
                     prometheus_collector = 1;
             } else {
                 if (EXPORTING_OPTIONS_DATA_SOURCE(exporting_options) == EXPORTING_SOURCE_DATA_AVERAGE &&
                     !(output_options & PROMETHEUS_OUTPUT_HIDEUNITS))
                     prometheus_units_copy(
-                        units, st->units, PROMETHEUS_ELEMENT_MAX, output_options & PROMETHEUS_OUTPUT_OLDUNITS);
+                        units, rrdset_units(st), PROMETHEUS_ELEMENT_MAX, output_options & PROMETHEUS_OUTPUT_OLDUNITS);
             }
 
             if (unlikely(output_options & PROMETHEUS_OUTPUT_HELP))
@@ -643,15 +600,14 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(
                     wb,
                     "\n# COMMENT %s chart \"%s\", context \"%s\", family \"%s\", units \"%s\"\n",
                     (homogeneous) ? "homogeneous" : "heterogeneous",
-                    (output_options & PROMETHEUS_OUTPUT_NAMES && st->name) ? st->name : st->id,
-                    st->context,
-                    st->family,
-                    st->units);
+                    (output_options & PROMETHEUS_OUTPUT_NAMES && st->name) ? rrdset_name(st) : rrdset_id(st),
+                    rrdset_context(st),
+                    rrdset_family(st),
+                    rrdset_units(st));
 
             // for each dimension
             RRDDIM *rd;
-            rrddim_foreach_read(rd, st)
-            {
+            rrddim_foreach_read(rd, st) {
                 if (rd->collections_counter && !rrddim_flag_check(rd, RRDDIM_FLAG_OBSOLETE)) {
                     char dimension[PROMETHEUS_ELEMENT_MAX + 1];
                     char *suffix = "";
@@ -690,7 +646,7 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(
 
                             prometheus_label_copy(
                                 dimension,
-                                (output_options & PROMETHEUS_OUTPUT_NAMES && rd->name) ? rd->name : rd->id,
+                                (output_options & PROMETHEUS_OUTPUT_NAMES && rd->name) ? rrddim_name(rd) : rrddim_id(rd),
                                 PROMETHEUS_ELEMENT_MAX);
 
                             if (unlikely(output_options & PROMETHEUS_OUTPUT_HELP))
@@ -700,13 +656,14 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(
                                 buffer_sprintf(wb, "# TYPE %s_%s%s %s\n", prefix, context, suffix, p.type);
 
                             generate_as_collected_prom_metric(wb, &p, homogeneous, prometheus_collector);
-                        } else {
+                        }
+                        else {
                             // the dimensions of the chart, do not have the same algorithm, multiplier or divisor
                             // we create a metric per dimension
 
                             prometheus_name_copy(
                                 dimension,
-                                (output_options & PROMETHEUS_OUTPUT_NAMES && rd->name) ? rd->name : rd->id,
+                                (output_options & PROMETHEUS_OUTPUT_NAMES && rd->name) ? rrddim_name(rd) : rrddim_id(rd),
                                 PROMETHEUS_ELEMENT_MAX);
 
                             if (unlikely(output_options & PROMETHEUS_OUTPUT_HELP))
@@ -718,12 +675,13 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(
 
                             generate_as_collected_prom_metric(wb, &p, homogeneous, prometheus_collector);
                         }
-                    } else {
+                    }
+                    else {
                         // we need average or sum of the data
 
                         time_t first_time = instance->after;
                         time_t last_time = instance->before;
-                        calculated_number value = exporting_calculate_value_from_stored_data(instance, rd, &last_time);
+                        NETDATA_DOUBLE value = exporting_calculate_value_from_stored_data(instance, rd, &last_time);
 
                         if (!isnan(value) && !isinf(value)) {
                             if (EXPORTING_OPTIONS_DATA_SOURCE(exporting_options) == EXPORTING_SOURCE_DATA_AVERAGE)
@@ -733,7 +691,7 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(
 
                             prometheus_label_copy(
                                 dimension,
-                                (output_options & PROMETHEUS_OUTPUT_NAMES && rd->name) ? rd->name : rd->id,
+                                (output_options & PROMETHEUS_OUTPUT_NAMES && rd->name) ? rrddim_name(rd) : rrddim_id(rd),
                                 PROMETHEUS_ELEMENT_MAX);
 
                             if (unlikely(output_options & PROMETHEUS_OUTPUT_HELP))
@@ -744,8 +702,8 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(
                                     context,
                                     units,
                                     suffix,
-                                    (output_options & PROMETHEUS_OUTPUT_NAMES && rd->name) ? rd->name : rd->id,
-                                    st->units,
+                                    (output_options & PROMETHEUS_OUTPUT_NAMES && rd->name) ? rrddim_name(rd) : rrddim_id(rd),
+                                    rrdset_units(st),
                                     (unsigned long long)first_time,
                                     (unsigned long long)last_time);
 
@@ -755,7 +713,7 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(
                             if (output_options & PROMETHEUS_OUTPUT_TIMESTAMPS)
                                 buffer_sprintf(
                                     wb,
-                                    "%s_%s%s%s{chart=\"%s\",family=\"%s\",dimension=\"%s\"%s} " CALCULATED_NUMBER_FORMAT
+                                    "%s_%s%s%s{chart=\"%s\",family=\"%s\",dimension=\"%s\"%s} " NETDATA_DOUBLE_FORMAT
                                     " %llu\n",
                                     prefix,
                                     context,
@@ -770,7 +728,7 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(
                             else
                                 buffer_sprintf(
                                     wb,
-                                    "%s_%s%s%s{chart=\"%s\",family=\"%s\",dimension=\"%s\"%s} " CALCULATED_NUMBER_FORMAT
+                                    "%s_%s%s%s{chart=\"%s\",family=\"%s\",dimension=\"%s\"%s} " NETDATA_DOUBLE_FORMAT
                                     "\n",
                                     prefix,
                                     context,
@@ -785,12 +743,11 @@ static void rrd_stats_api_v1_charts_allmetrics_prometheus(
                     }
                 }
             }
-
-            rrdset_unlock(st);
+            rrddim_foreach_done(rd);
         }
     }
+    rrdset_foreach_done(st);
 
-    rrdhost_unlock(host);
     simple_pattern_free(filter);
 }
 
@@ -848,7 +805,7 @@ static inline time_t prometheus_preparation(
         buffer_sprintf(
             wb,
             "# COMMENT netdata \"%s\" to %sprometheus \"%s\", source \"%s\", last seen %lu %s, time range %lu to %lu\n\n",
-            host->hostname,
+            rrdhost_hostname(host),
             (first_seen) ? "FIRST SEEN " : "",
             server,
             mode,
