@@ -97,13 +97,14 @@ int init_prometheus_remote_write_instance(struct instance *instance)
     instance->start_chart_formatting = format_chart_prometheus_remote_write;
     instance->metric_formatting = format_dimension_prometheus_remote_write;
     instance->end_chart_formatting = NULL;
+    instance->variables_formatting = format_variables_prometheus_remote_write;
     instance->end_host_formatting = NULL;
     instance->end_batch_formatting = format_batch_prometheus_remote_write;
 
     instance->prepare_header = prometheus_remote_write_prepare_header;
     instance->check_response = process_prometheus_remote_write_response;
 
-    instance->buffer = (void *)buffer_create(0);
+    instance->buffer = (void *)buffer_create(0, &netdata_buffers_statistics.buffers_exporters);
 
     if (uv_mutex_init(&instance->mutex))
         return 1;
@@ -134,6 +135,25 @@ int init_prometheus_remote_write_instance(struct instance *instance)
     return 0;
 }
 
+struct format_remote_write_label_callback {
+    struct instance *instance;
+    void *write_request;
+};
+
+static int format_remote_write_label_callback(const char *name, const char *value, RRDLABEL_SRC ls, void *data) {
+    struct format_remote_write_label_callback *d = (struct format_remote_write_label_callback *)data;
+
+    if (!should_send_label(d->instance, ls)) return 0;
+
+    char k[PROMETHEUS_ELEMENT_MAX + 1];
+    char v[PROMETHEUS_ELEMENT_MAX + 1];
+
+    prometheus_name_copy(k, name, PROMETHEUS_ELEMENT_MAX);
+    prometheus_label_copy(v, value, PROMETHEUS_ELEMENT_MAX);
+    add_label(d->write_request, k, v);
+    return 1;
+}
+
 /**
  * Format host data for Prometheus Remote Write connector
  *
@@ -151,29 +171,19 @@ int format_host_prometheus_remote_write(struct instance *instance, RRDHOST *host
     char hostname[PROMETHEUS_ELEMENT_MAX + 1];
     prometheus_label_copy(
         hostname,
-        (host == localhost) ? instance->config.hostname : host->hostname,
+        (host == localhost) ? instance->config.hostname : rrdhost_hostname(host),
         PROMETHEUS_ELEMENT_MAX);
 
     add_host_info(
         connector_specific_data->write_request,
-        "netdata_info", hostname, host->program_name, host->program_version, now_realtime_usec() / USEC_PER_MS);
-
+        "netdata_info", hostname, rrdhost_program_name(host), rrdhost_program_version(host), now_realtime_usec() / USEC_PER_MS);
+    
     if (unlikely(sending_labels_configured(instance))) {
-        rrdhost_check_rdlock(host);
-        netdata_rwlock_rdlock(&host->labels.labels_rwlock);
-        for (struct label *label = host->labels.head; label; label = label->next) {
-            if (!should_send_label(instance, label))
-                continue;
-
-            char key[PROMETHEUS_ELEMENT_MAX + 1];
-            prometheus_name_copy(key, label->key, PROMETHEUS_ELEMENT_MAX);
-
-            char value[PROMETHEUS_ELEMENT_MAX + 1];
-            prometheus_label_copy(value, label->value, PROMETHEUS_ELEMENT_MAX);
-
-            add_label(connector_specific_data->write_request, key, value);
-        }
-        netdata_rwlock_unlock(&host->labels.labels_rwlock);
+        struct format_remote_write_label_callback tmp = {
+            .write_request = connector_specific_data->write_request,
+            .instance = instance
+        };
+        rrdlabels_walkthrough_read(host->rrdlabels, format_remote_write_label_callback, &tmp);
     }
 
     return 0;
@@ -190,10 +200,10 @@ int format_chart_prometheus_remote_write(struct instance *instance, RRDSET *st)
 {
     prometheus_label_copy(
         chart,
-        (instance->config.options & EXPORTING_OPTION_SEND_NAMES && st->name) ? st->name : st->id,
+        (instance->config.options & EXPORTING_OPTION_SEND_NAMES && st->name) ? rrdset_name(st) : rrdset_id(st),
         PROMETHEUS_ELEMENT_MAX);
-    prometheus_label_copy(family, st->family, PROMETHEUS_ELEMENT_MAX);
-    prometheus_name_copy(context, st->context, PROMETHEUS_ELEMENT_MAX);
+    prometheus_label_copy(family, rrdset_family(st), PROMETHEUS_ELEMENT_MAX);
+    prometheus_name_copy(context, rrdset_context(st), PROMETHEUS_ELEMENT_MAX);
 
     as_collected = (EXPORTING_OPTIONS_DATA_SOURCE(instance->config.options) == EXPORTING_SOURCE_DATA_AS_COLLECTED);
     homogeneous = 1;
@@ -205,7 +215,7 @@ int format_chart_prometheus_remote_write(struct instance *instance, RRDSET *st)
             homogeneous = 0;
     } else {
         if (EXPORTING_OPTIONS_DATA_SOURCE(instance->config.options) == EXPORTING_SOURCE_DATA_AVERAGE)
-            prometheus_units_copy(units, st->units, PROMETHEUS_ELEMENT_MAX, 0);
+            prometheus_units_copy(units, rrdset_units(st), PROMETHEUS_ELEMENT_MAX, 0);
     }
 
     return 0;
@@ -239,28 +249,33 @@ int format_dimension_prometheus_remote_write(struct instance *instance, RRDDIM *
                     D_EXPORTING,
                     "EXPORTING: not sending dimension '%s' of chart '%s' from host '%s', "
                     "its last data collection (%lu) is not within our timeframe (%lu to %lu)",
-                    rd->id, rd->rrdset->id,
-                    (host == localhost) ? instance->config.hostname : host->hostname,
+                    rrddim_id(rd), rrdset_id(rd->rrdset),
+                    (host == localhost) ? instance->config.hostname : rrdhost_hostname(host),
                     (unsigned long)rd->last_collected_time.tv_sec,
                     (unsigned long)instance->after,
                     (unsigned long)instance->before);
                 return 0;
             }
 
+            if (rd->algorithm == RRD_ALGORITHM_INCREMENTAL || rd->algorithm == RRD_ALGORITHM_PCENT_OVER_DIFF_TOTAL) {
+                if (strcmp(rrdset_module_name(rd->rrdset), "prometheus"))
+                    suffix = "_total";
+            }
+
             if (homogeneous) {
                 // all the dimensions of the chart, has the same algorithm, multiplier and divisor
                 // we add all dimensions as labels
 
                 prometheus_label_copy(
                     dimension,
-                    (instance->config.options & EXPORTING_OPTION_SEND_NAMES && rd->name) ? rd->name : rd->id,
+                    (instance->config.options & EXPORTING_OPTION_SEND_NAMES && rd->name) ? rrddim_name(rd) : rrddim_id(rd),
                     PROMETHEUS_ELEMENT_MAX);
                 snprintf(name, PROMETHEUS_LABELS_MAX, "%s_%s%s", instance->config.prefix, context, suffix);
 
                 add_metric(
                     connector_specific_data->write_request,
                     name, chart, family, dimension,
-                    (host == localhost) ? instance->config.hostname : host->hostname,
+                    (host == localhost) ? instance->config.hostname : rrdhost_hostname(host),
                     rd->last_collected_value, timeval_msec(&rd->last_collected_time));
             } else {
                 // the dimensions of the chart, do not have the same algorithm, multiplier or divisor
@@ -268,7 +283,7 @@ int format_dimension_prometheus_remote_write(struct instance *instance, RRDDIM *
 
                 prometheus_name_copy(
                     dimension,
-                    (instance->config.options & EXPORTING_OPTION_SEND_NAMES && rd->name) ? rd->name : rd->id,
+                    (instance->config.options & EXPORTING_OPTION_SEND_NAMES && rd->name) ? rrddim_name(rd) : rrddim_id(rd),
                     PROMETHEUS_ELEMENT_MAX);
                 snprintf(
                     name, PROMETHEUS_LABELS_MAX, "%s_%s_%s%s", instance->config.prefix, context, dimension,
@@ -277,14 +292,14 @@ int format_dimension_prometheus_remote_write(struct instance *instance, RRDDIM *
                 add_metric(
                     connector_specific_data->write_request,
                     name, chart, family, NULL,
-                    (host == localhost) ? instance->config.hostname : host->hostname,
+                    (host == localhost) ? instance->config.hostname : rrdhost_hostname(host),
                     rd->last_collected_value, timeval_msec(&rd->last_collected_time));
             }
         } else {
             // we need average or sum of the data
 
             time_t last_t = instance->before;
-            calculated_number value = exporting_calculate_value_from_stored_data(instance, rd, &last_t);
+            NETDATA_DOUBLE value = exporting_calculate_value_from_stored_data(instance, rd, &last_t);
 
             if (!isnan(value) && !isinf(value)) {
                 if (EXPORTING_OPTIONS_DATA_SOURCE(instance->config.options) == EXPORTING_SOURCE_DATA_AVERAGE)
@@ -294,7 +309,7 @@ int format_dimension_prometheus_remote_write(struct instance *instance, RRDDIM *
 
                 prometheus_label_copy(
                     dimension,
-                    (instance->config.options & EXPORTING_OPTION_SEND_NAMES && rd->name) ? rd->name : rd->id,
+                    (instance->config.options & EXPORTING_OPTION_SEND_NAMES && rd->name) ? rrddim_name(rd) : rrddim_id(rd),
                     PROMETHEUS_ELEMENT_MAX);
                 snprintf(
                     name, PROMETHEUS_LABELS_MAX, "%s_%s%s%s", instance->config.prefix, context, units, suffix);
@@ -302,7 +317,7 @@ int format_dimension_prometheus_remote_write(struct instance *instance, RRDDIM *
                 add_metric(
                     connector_specific_data->write_request,
                     name, chart, family, dimension,
-                    (host == localhost) ? instance->config.hostname : host->hostname,
+                    (host == localhost) ? instance->config.hostname : rrdhost_hostname(host),
                     value, last_t * MSEC_PER_SEC);
             }
         }
@@ -311,6 +326,51 @@ int format_dimension_prometheus_remote_write(struct instance *instance, RRDDIM *
     return 0;
 }
 
+static int format_variable_prometheus_remote_write_callback(const DICTIONARY_ITEM *item __maybe_unused, void *rv_ptr __maybe_unused, void *data) {
+    const RRDVAR_ACQUIRED *rv = (const RRDVAR_ACQUIRED *)item;
+
+    struct prometheus_remote_write_variables_callback_options *opts = data;
+
+    if (rrdvar_flags(rv) & (RRDVAR_FLAG_CUSTOM_HOST_VAR | RRDVAR_FLAG_CUSTOM_CHART_VAR)) {
+        RRDHOST *host = opts->host;
+        struct instance *instance = opts->instance;
+        struct simple_connector_data *simple_connector_data =
+            (struct simple_connector_data *)instance->connector_specific_data;
+        struct prometheus_remote_write_specific_data *connector_specific_data =
+            (struct prometheus_remote_write_specific_data *)simple_connector_data->connector_specific_data;
+
+        char name[PROMETHEUS_LABELS_MAX + 1];
+        char *suffix = "";
+
+        prometheus_name_copy(context, rrdvar_name(rv), PROMETHEUS_ELEMENT_MAX);
+        snprintf(name, PROMETHEUS_LABELS_MAX, "%s_%s%s", instance->config.prefix, context, suffix);
+
+        NETDATA_DOUBLE value = rrdvar2number(rv);
+        add_variable(connector_specific_data->write_request, name,
+            (host == localhost) ? instance->config.hostname : rrdhost_hostname(host), value, opts->now / USEC_PER_MS);
+    }
+
+    return 0;
+}
+
+/**
+ * Format a variable for Prometheus Remote Write connector
+ * 
+ * @param rv a variable.
+ * @param instance an instance data structure.
+ * @return Always returns 0.
+ */ 
+int format_variables_prometheus_remote_write(struct instance *instance, RRDHOST *host)
+{
+    struct prometheus_remote_write_variables_callback_options opt = {
+        .host = host,
+        .instance = instance,
+        .now = now_realtime_usec(),
+    };
+
+    return rrdvar_walkthrough_read(host->rrdvars, format_variable_prometheus_remote_write_callback, &opt);
+}
+
 /**
  * Format a batch for Prometheus Remote Write connector
  *
@@ -339,7 +399,6 @@ int format_batch_prometheus_remote_write(struct instance *instance)
         return 1;
     }
     buffer->len = data_size;
-    instance->stats.buffered_bytes = (collected_number)buffer_strlen(buffer);
 
     simple_connector_end_batch(instance);
 
