@@ -60,13 +60,10 @@ typedef void (*msg_callback_fnc_t)(const char *topic, const void *msg, size_t ms
 mqtt_wss_client mqtt_wss_new(const char *log_prefix,
                              mqtt_wss_log_callback_t log_callback,
                              msg_callback_fnc_t msg_callback,
-                             void (*puback_callback)(uint16_t packet_id),
-                             int mqtt5);
+                             void (*puback_callback)(uint16_t packet_id));
 
 void mqtt_wss_set_max_buf_size(mqtt_wss_client client, size_t size);
 
-int mqtt_wss_able_to_send(mqtt_wss_client client, size_t bytes);
-
 void mqtt_wss_destroy(mqtt_wss_client client);
 
 struct mqtt_connect_params;
@@ -118,19 +115,12 @@ struct mqtt_wss_proxy {
     enum mqtt_wss_proxy_type type;
     const char *host;
     int port;
+    const char *username;
+    const char *password;
 };
 
-/* Publishes MQTT message
- * @param client mqtt_wss_client which should transfer the message
- * @param topic MQTT topic to publish message to (0 terminated C string)
- * @param msg Message to be published (no need for 0 termination)
- * @param msg_len Length of the message to be published
- * @param publish_flags see enum mqtt_wss_publish_flags e.g. (MQTT_WSS_PUB_QOS1 | MQTT_WSS_PUB_RETAIN)
- * @return Returns 0 on success
- */
-int mqtt_wss_publish(mqtt_wss_client client, const char *topic, const void *msg, int msg_len, uint8_t publish_flags);
-
-/* Publishes MQTT message and gets message id
+/* TODO!!! update the description
+ * Publishes MQTT message and gets message id
  * @param client mqtt_wss_client which should transfer the message
  * @param topic MQTT topic to publish message to (0 terminated C string)
  * @param msg Message to be published (no need for 0 termination)
@@ -140,12 +130,6 @@ int mqtt_wss_publish(mqtt_wss_client client, const char *topic, const void *msg,
  *        for usages where application layer wants to know which messages are delivered when
  * @return Returns 0 on success
  */
-int mqtt_wss_publish_pid(mqtt_wss_client client, const char *topic, const void *msg, int msg_len, uint8_t publish_flags, uint16_t *packet_id);
-
-int mqtt_wss_publish_pid_block(mqtt_wss_client client, const char *topic, const void *msg, int msg_len, uint8_t publish_flags, uint16_t *packet_id, int timeout_ms);
-
-/* Publishes MQTT 5 message
- */
 int mqtt_wss_publish5(mqtt_wss_client client,
                       char *topic,
                       free_fnc_t topic_free,
@@ -155,6 +139,8 @@ int mqtt_wss_publish5(mqtt_wss_client client,
                       uint8_t publish_flags,
                       uint16_t *packet_id);
 
+int mqtt_wss_set_topic_alias(mqtt_wss_client client, const char *topic);
+
 /* Subscribes to MQTT topic
  * @param client mqtt_wss_client which should do the subscription
  * @param topic MQTT topic to subscribe to
@@ -163,4 +149,25 @@ int mqtt_wss_publish5(mqtt_wss_client client,
  */
 int mqtt_wss_subscribe(mqtt_wss_client client, char *topic, int max_qos_level);
 
+
+struct mqtt_wss_stats {
+    uint64_t bytes_tx;
+    uint64_t bytes_rx;
+#ifdef MQTT_WSS_CPUSTATS
+    uint64_t time_keepalive;
+    uint64_t time_read_socket;
+    uint64_t time_write_socket;
+    uint64_t time_process_websocket;
+    uint64_t time_process_mqtt;
+#endif
+    struct mqtt_ng_stats mqtt;
+};
+
+struct mqtt_wss_stats mqtt_wss_get_stats(mqtt_wss_client client);
+
+#ifdef MQTT_WSS_DEBUG
+#include <openssl/ssl.h>
+void mqtt_wss_set_SSL_CTX_keylog_cb(mqtt_wss_client client, void (*ssl_ctx_keylog_cb)(const SSL *ssl, const char *line));
+#endif
+
 #endif /* MQTT_WSS_CLIENT_H */
