@@ -79,7 +79,7 @@ _cannot_use_tmpdir() {
 if [ -z "${TMPDIR}" ] || _cannot_use_tmpdir "${TMPDIR}"; then
   if _cannot_use_tmpdir /tmp; then
     if _cannot_use_tmpdir "${PWD}"; then
-      fatal "UUnable to find a usable temporary directory. Please set \$TMPDIR to a path that is both writable and allows execution of files and try again." I0000
+      fatal "Unable to find a usable temporary directory. Please set \$TMPDIR to a path that is both writable and allows execution of files and try again." I0000
     else
       TMPDIR="${PWD}"
     fi
@@ -163,10 +163,6 @@ banner_nonroot_install() {
 
       $PROGRAM ${@} --install-prefix /tmp
 
-  or
-
-      $PROGRAM ${@} --install /tmp
-
   or, run the installer as root:
 
       sudo $PROGRAM ${@}
@@ -203,8 +199,7 @@ usage() {
 USAGE: ${PROGRAM} [options]
        where options include:
 
-  --install <path>           Install netdata in <path>. Ex. --install /opt will put netdata in /opt/netdata, this option is deprecated and will be removed in a future version, please use --install-prefix instead.
-  --install-prefix <path>           Install netdata in <path>. Ex. --install-prefix /opt will put netdata in /opt/netdata.
+  --install-prefix <path>    Install netdata in <path>. Ex. --install-prefix /opt will put netdata in /opt/netdata.
   --dont-start-it            Do not (re)start netdata after installation.
   --dont-wait                Run installation in non-interactive mode.
   --stable-channel           Use packages from GitHub release pages instead of nightly updates.
@@ -239,7 +234,6 @@ USAGE: ${PROGRAM} [options]
   --enable-ml                Enable anomaly detection with machine learning. Default: autodetect.
   --disable-ml               Explicitly disable anomaly detection with machine learning.
   --disable-x86-sse          Disable SSE instructions & optimizations. Default: enabled.
-  --use-system-lws           Use a system copy of libwebsockets instead of bundled copy. Default: bundled.
   --use-system-protobuf      Use a system copy of libprotobuf instead of bundled copy. Default: bundled.
   --zlib-is-really-here
   --libs-are-really-here     If you see errors about missing zlib or libuuid but you know it is available, you might
@@ -286,7 +280,7 @@ NETDATA_PREFIX=
 LIBS_ARE_HERE=0
 NETDATA_ENABLE_ML=""
 NETDATA_CONFIGURE_OPTIONS="${NETDATA_CONFIGURE_OPTIONS-}"
-RELEASE_CHANNEL="nightly" # valid values are 'nightly' and 'stable'
+RELEASE_CHANNEL="stable" # valid values are 'nightly' and 'stable'
 IS_NETDATA_STATIC_BINARY="${IS_NETDATA_STATIC_BINARY:-"no"}"
 while [ -n "${1}" ]; do
   case "${1}" in
@@ -308,7 +302,6 @@ while [ -n "${1}" ]; do
     "--disable-https") NETDATA_CONFIGURE_OPTIONS="$(echo "${NETDATA_CONFIGURE_OPTIONS%--disable-https)}" | sed 's/$/ --disable-plugin-https/g')" ;;
     "--disable-dbengine")
       NETDATA_CONFIGURE_OPTIONS="$(echo "${NETDATA_CONFIGURE_OPTIONS%--disable-dbengine)}" | sed 's/$/ --disable-dbengine/g')"
-      NETDATA_DISABLE_DBENGINE=1
       ;;
     "--enable-plugin-nfacct") NETDATA_CONFIGURE_OPTIONS="$(echo "${NETDATA_CONFIGURE_OPTIONS%--enable-plugin-nfacct)}" | sed 's/$/ --enable-plugin-nfacct/g')" ;;
     "--disable-plugin-nfacct") NETDATA_CONFIGURE_OPTIONS="$(echo "${NETDATA_CONFIGURE_OPTIONS%--disable-plugin-nfacct)}" | sed 's/$/ --disable-plugin-nfacct/g')" ;;
@@ -339,8 +332,6 @@ while [ -n "${1}" ]; do
       NETDATA_CONFIGURE_OPTIONS="$(echo "${NETDATA_CONFIGURE_OPTIONS%--disable-ml)}" | sed 's/$/ --disable-ml/g')"
       NETDATA_ENABLE_ML=0
       ;;
-    "--enable-ml-tests") NETDATA_CONFIGURE_OPTIONS="$(echo "${NETDATA_CONFIGURE_OPTIONS%--enable-ml-tests)}" | sed 's/$/ --enable-ml-tests/g')" ;;
-    "--disable-ml-tests") NETDATA_CONFIGURE_OPTIONS="$(echo "${NETDATA_CONFIGURE_OPTIONS%--disable-ml-tests)}" | sed 's/$/ --disable-ml-tests/g')" ;;
     "--disable-lto") NETDATA_CONFIGURE_OPTIONS="$(echo "${NETDATA_CONFIGURE_OPTIONS%--disable-lto)}" | sed 's/$/ --disable-lto/g')" ;;
     "--disable-x86-sse") NETDATA_CONFIGURE_OPTIONS="$(echo "${NETDATA_CONFIGURE_OPTIONS%--disable-x86-sse)}" | sed 's/$/ --disable-x86-sse/g')" ;;
     "--disable-telemetry") NETDATA_DISABLE_TELEMETRY=1 ;;
@@ -368,13 +359,6 @@ while [ -n "${1}" ]; do
     "--build-json-c")
       NETDATA_BUILD_JSON_C=1
       ;;
-    "--build-judy")
-      NETDATA_BUILD_JUDY=1
-      ;;
-    "--install")
-      NETDATA_PREFIX="${2}/netdata"
-      shift 1
-      ;;
     "--install-prefix")
       NETDATA_PREFIX="${2}/netdata"
       shift 1
@@ -445,7 +429,7 @@ if [ "$(uname -s)" = "Linux" ] && [ -f /proc/meminfo ]; then
       target_ram="$(echo "${target_ram}" | awk '{$1/=1024*1024*1024;printf "%.2fGiB\n",$1}')"
       total_ram="$(echo "${total_ram}" | awk '{$1/=1024*1024*1024;printf "%.2fGiB\n",$1}')"
       run_failed "Netdata needs ${target_ram} of RAM to safely install, but this system only has ${total_ram}. Try reducing the number of processes used for the install using the \$MAKEOPTS variable."
-      exit_reason "Insufficent RAM to safely install." I000F
+      exit_reason "Insufficient RAM to safely install." I000F
       exit 2
     fi
   fi
@@ -659,98 +643,6 @@ bundle_protobuf() {
 
 bundle_protobuf
 
-# -----------------------------------------------------------------------------
-
-build_judy() {
-  env_cmd=''
-  libtoolize="libtoolize"
-
-  if [ -z "${DONT_SCRUB_CFLAGS_EVEN_THOUGH_IT_MAY_BREAK_THINGS}" ]; then
-    env_cmd="env CFLAGS='-fPIC -pipe' CXXFLAGS='-fPIC -pipe' LDFLAGS="
-  fi
-
-  if [ "$(uname)" = "Darwin" ]; then
-    libtoolize="glibtoolize"
-  fi
-
-  cd "${1}" > /dev/null || return 1
-  if run eval "${env_cmd} ${libtoolize} --force --copy" &&
-    run eval "${env_cmd} aclocal" &&
-    run eval "${env_cmd} autoheader" &&
-    run eval "${env_cmd} automake --add-missing --force --copy --include-deps" &&
-    run eval "${env_cmd} autoconf" &&
-    run eval "${env_cmd} ./configure" &&
-    run eval "${env_cmd} ${make} ${MAKEOPTS} -C src" &&
-    run eval "${env_cmd} ar -r src/libJudy.a src/Judy*/*.o"; then
-    cd - > /dev/null || return 1
-  else
-    cd - > /dev/null || return 1
-    return 1
-  fi
-}
-
-copy_judy() {
-  target_dir="${PWD}/externaldeps/libJudy"
-
-  run mkdir -p "${target_dir}" || return 1
-
-  run cp "${1}/src/libJudy.a" "${target_dir}/libJudy.a" || return 1
-  run cp "${1}/src/Judy.h" "${target_dir}/Judy.h" || return 1
-}
-
-bundle_judy() {
-  # If --build-judy flag or no Judy on the system and we're building the dbengine, bundle our own libJudy.
-  # shellcheck disable=SC2235,SC2030,SC2031
-  if [ -n "${NETDATA_DISABLE_DBENGINE}" ] || ([ -z "${NETDATA_BUILD_JUDY}" ] && [ -e /usr/include/Judy.h ]); then
-    return 0
-  elif [ -n "${NETDATA_BUILD_JUDY}" ]; then
-    progress "User requested bundling of libJudy, building it now"
-  elif [ ! -e /usr/include/Judy.h ]; then
-    progress "/usr/include/Judy.h does not exist, but we need libJudy, building our own copy"
-  fi
-
-  [ -n "${GITHUB_ACTIONS}" ] && echo "::group::Bundling libJudy."
-
-  progress "Prepare libJudy"
-
-  JUDY_PACKAGE_VERSION="$(cat packaging/judy.version)"
-
-  tmp="$(mktemp -d -t netdata-judy-XXXXXX)"
-  JUDY_PACKAGE_BASENAME="v${JUDY_PACKAGE_VERSION}.tar.gz"
-
-  if fetch_and_verify "judy" \
-    "https://github.com/netdata/libjudy/archive/${JUDY_PACKAGE_BASENAME}" \
-    "${JUDY_PACKAGE_BASENAME}" \
-    "${tmp}" \
-    "${NETDATA_LOCAL_TARBALL_OVERRIDE_JUDY}"; then
-    if run tar --no-same-owner -xf "${tmp}/${JUDY_PACKAGE_BASENAME}" -C "${tmp}" &&
-      build_judy "${tmp}/libjudy-${JUDY_PACKAGE_VERSION}" &&
-      copy_judy "${tmp}/libjudy-${JUDY_PACKAGE_VERSION}" &&
-      rm -rf "${tmp}"; then
-      run_ok "libJudy built and prepared."
-      NETDATA_CONFIGURE_OPTIONS="${NETDATA_CONFIGURE_OPTIONS} --with-bundled-libJudy"
-    else
-      if [ -n "${NETDATA_BUILD_JUDY}" ]; then
-        [ -n "${GITHUB_ACTIONS}" ] && echo "::endgroup::"
-        fatal "failed to build libJudy." I0003
-      else
-        run_failed "Failed to build libJudy, dbengine support will be disabled."
-      fi
-    fi
-  else
-    if [ -n "${NETDATA_BUILD_JUDY}" ]; then
-      [ -n "${GITHUB_ACTIONS}" ] && echo "::endgroup::"
-      fatal "Unable to fetch sources for libJudy, which is required for this build of Netdata." I0004
-    else
-      run_failed "Unable to fetch sources for libJudy, which is required for this build of Netdata."
-    fi
-  fi
-
-  [ -n "${GITHUB_ACTIONS}" ] && echo "::endgroup::"
-}
-
-bundle_judy
-
 # -----------------------------------------------------------------------------
 build_jsonc() {
   env_cmd=''
@@ -844,14 +736,11 @@ detect_libc() {
     echo >&2 " Detected musl"
     libc="musl"
   else
-    ls_path=$(command -v ls)
-    if [ -n "${ls_path}" ] ; then
-      cmd=$(ldd "$ls_path" | grep -w libc | cut -d" " -f 3)
+      cmd=$(ldd /bin/sh | grep -w libc | cut -d" " -f 3)
       if bash -c "${cmd}" 2>&1 | grep -q -i "GNU C Library"; then
         echo >&2 " Detected GLIBC"
         libc="glibc"
       fi
-    fi
   fi
 
   if [ -z "$libc" ]; then
@@ -1034,6 +923,34 @@ if [ "$have_autotools" ]; then
   fi
 fi
 
+# function to extract values from the config file
+config_option() {
+  section="${1}"
+  key="${2}"
+  value="${3}"
+
+  if [ -x "${NETDATA_PREFIX}/usr/sbin/netdata" ] && [ -r "${NETDATA_PREFIX}/etc/netdata/netdata.conf" ]; then
+    "${NETDATA_PREFIX}/usr/sbin/netdata" \
+      -c "${NETDATA_PREFIX}/etc/netdata/netdata.conf" \
+      -W get "${section}" "${key}" "${value}" ||
+      echo "${value}"
+  else
+    echo "${value}"
+  fi
+}
+
+# the user netdata will run as
+if [ "$(id -u)" = "0" ]; then
+  NETDATA_USER="$(config_option "global" "run as user" "netdata")"
+  ROOT_USER="root"
+else
+  NETDATA_USER="${USER}"
+  ROOT_USER="${USER}"
+fi
+NETDATA_GROUP="$(id -g -n "${NETDATA_USER}")"
+[ -z "${NETDATA_GROUP}" ] && NETDATA_GROUP="${NETDATA_USER}"
+echo >&2 "Netdata user and group set to: ${NETDATA_USER}/${NETDATA_GROUP}"
+
 # shellcheck disable=SC2086
 if ! run ./configure \
          --prefix="${NETDATA_PREFIX}/usr" \
@@ -1043,7 +960,7 @@ if ! run ./configure \
          --libdir="${NETDATA_PREFIX}/usr/lib" \
          --with-zlib \
          --with-math \
-         --with-user=netdata \
+         --with-user="${NETDATA_USER}" \
          ${NETDATA_CONFIGURE_OPTIONS} \
          CFLAGS="${CFLAGS}" LDFLAGS="${LDFLAGS}"; then
   fatal "Failed to configure Netdata sources." I000A
@@ -1161,6 +1078,11 @@ if [ "$(id -u)" -eq 0 ]; then
     # shellcheck disable=SC2086
     portable_add_user_to_group ${g} netdata && NETDATA_ADDED_TO_GROUPS="${NETDATA_ADDED_TO_GROUPS} ${g}"
   done
+  # Netdata must be able to read /etc/pve/qemu-server/* and /etc/pve/lxc/* 
+  # for reading VMs/containers names, CPU and memory limits on Proxmox.
+  if [ -d "/etc/pve" ]; then
+    portable_add_user_to_group "www-data" netdata && NETDATA_ADDED_TO_GROUPS="${NETDATA_ADDED_TO_GROUPS} www-data"
+  fi
 else
   run_failed "The installer does not run as root. Nothing to do for user and groups"
 fi
@@ -1177,34 +1099,6 @@ progress "Read installation options from netdata.conf"
 [ ! -f "${NETDATA_PREFIX}/etc/netdata/netdata.conf" ] &&
   touch "${NETDATA_PREFIX}/etc/netdata/netdata.conf"
 
-# function to extract values from the config file
-config_option() {
-  section="${1}"
-  key="${2}"
-  value="${3}"
-
-  if [ -s "${NETDATA_PREFIX}/etc/netdata/netdata.conf" ]; then
-    "${NETDATA_PREFIX}/usr/sbin/netdata" \
-      -c "${NETDATA_PREFIX}/etc/netdata/netdata.conf" \
-      -W get "${section}" "${key}" "${value}" ||
-      echo "${value}"
-  else
-    echo "${value}"
-  fi
-}
-
-# the user netdata will run as
-if [ "$(id -u)" = "0" ]; then
-  NETDATA_USER="$(config_option "global" "run as user" "netdata")"
-  ROOT_USER="root"
-else
-  NETDATA_USER="${USER}"
-  ROOT_USER="${USER}"
-fi
-NETDATA_GROUP="$(id -g -n "${NETDATA_USER}")"
-[ -z "${NETDATA_GROUP}" ] && NETDATA_GROUP="${NETDATA_USER}"
-echo >&2 "Netdata user and group is finally set to: ${NETDATA_USER}/${NETDATA_GROUP}"
-
 # port
 defport=19999
 NETDATA_PORT="$(config_option "web" "default port" ${defport})"
@@ -1306,8 +1200,8 @@ run chmod 770 "${NETDATA_CLAIMING_DIR}"
 if [ "$(id -u)" -eq 0 ]; then
   # find the admin group
   admin_group=
-  test -z "${admin_group}" && getent group root > /dev/null 2>&1 && admin_group="root"
-  test -z "${admin_group}" && getent group daemon > /dev/null 2>&1 && admin_group="daemon"
+  test -z "${admin_group}" && get_group root > /dev/null 2>&1 && admin_group="root"
+  test -z "${admin_group}" && get_group daemon > /dev/null 2>&1 && admin_group="daemon"
   test -z "${admin_group}" && admin_group="${NETDATA_GROUP}"
 
   run chown "${NETDATA_USER}:${admin_group}" "${NETDATA_LOG_DIR}"
@@ -1540,6 +1434,9 @@ install_go() {
     run chown "root:${NETDATA_GROUP}" "${NETDATA_PREFIX}/usr/libexec/netdata/plugins.d/go.d.plugin"
   fi
   run chmod 0750 "${NETDATA_PREFIX}/usr/libexec/netdata/plugins.d/go.d.plugin"
+  if command -v setcap 1>/dev/null 2>&1; then
+    run setcap "cap_net_admin+epi cap_net_raw=eip" "${NETDATA_PREFIX}/usr/libexec/netdata/plugins.d/go.d.plugin"
+  fi
   rm -rf "${tmp}"
 
   [ -n "${GITHUB_ACTIONS}" ] && echo "::endgroup::"
@@ -1859,7 +1756,6 @@ echo >&2
 
 # -----------------------------------------------------------------------------
 progress "Installing (but not enabling) the netdata updater tool"
-cleanup_old_netdata_updater || run_failed "Cannot cleanup old netdata updater tool."
 install_netdata_updater || run_failed "Cannot install netdata updater tool."
 
 # -----------------------------------------------------------------------------
