@@ -1,101 +1,7 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 
 #include "web/api/web_api_v1.h"
-
-static inline void free_single_rrdrim(ONEWAYALLOC *owa, RRDDIM *temp_rd, int archive_mode)
-{
-    if (unlikely(!temp_rd))
-        return;
-
-    onewayalloc_freez(owa, (char *)temp_rd->id);
-
-    if (unlikely(archive_mode)) {
-        temp_rd->rrdset->counter--;
-        if (!temp_rd->rrdset->counter) {
-            onewayalloc_freez(owa, (char *)temp_rd->rrdset->name);
-            onewayalloc_freez(owa, temp_rd->rrdset->context);
-            onewayalloc_freez(owa, temp_rd->rrdset);
-        }
-    }
-
-    onewayalloc_freez(owa, temp_rd->state);
-    onewayalloc_freez(owa, temp_rd);
-}
-
-static inline void free_rrddim_list(ONEWAYALLOC *owa, RRDDIM *temp_rd, int archive_mode)
-{
-    if (unlikely(!temp_rd))
-        return;
-
-    RRDDIM *t;
-    while (temp_rd) {
-        t = temp_rd->next;
-        free_single_rrdrim(owa, temp_rd, archive_mode);
-        temp_rd = t;
-    }
-}
-
-void free_context_param_list(ONEWAYALLOC *owa, struct context_param **param_list)
-{
-    if (unlikely(!param_list || !*param_list))
-        return;
-
-    free_rrddim_list(owa, ((*param_list)->rd), (*param_list)->flags & CONTEXT_FLAGS_ARCHIVE);
-    onewayalloc_freez(owa, (*param_list));
-    *param_list = NULL;
-}
-
-void rebuild_context_param_list(ONEWAYALLOC *owa, struct context_param *context_param_list, time_t after_requested)
-{
-    RRDDIM *temp_rd = context_param_list->rd;
-    RRDDIM *new_rd_list = NULL, *t;
-    int is_archived = (context_param_list->flags & CONTEXT_FLAGS_ARCHIVE);
-    while (temp_rd) {
-        t = temp_rd->next;
-        RRDSET *st = temp_rd->rrdset;
-        time_t last_entry_t = is_archived ? st->last_entry_t : rrdset_last_entry_t(st);
-
-        if (last_entry_t >= after_requested) {
-            temp_rd->next = new_rd_list;
-            new_rd_list = temp_rd;
-        } else
-            free_single_rrdrim(owa, temp_rd, is_archived);
-        temp_rd = t;
-    }
-    context_param_list->rd = new_rd_list;
-};
-
-void build_context_param_list(ONEWAYALLOC *owa, struct context_param **param_list, RRDSET *st)
-{
-    if (unlikely(!param_list || !st))
-        return;
-
-    if (unlikely(!(*param_list))) {
-        *param_list = onewayalloc_mallocz(owa, sizeof(struct context_param));
-        (*param_list)->first_entry_t = LONG_MAX;
-        (*param_list)->last_entry_t = 0;
-        (*param_list)->flags = CONTEXT_FLAGS_CONTEXT;
-        (*param_list)->rd = NULL;
-    }
-
-    RRDDIM *rd1;
-    st->last_accessed_time = now_realtime_sec();
-    rrdset_rdlock(st);
-
-    (*param_list)->first_entry_t = MIN((*param_list)->first_entry_t, rrdset_first_entry_t_nolock(st));
-    (*param_list)->last_entry_t  = MAX((*param_list)->last_entry_t, rrdset_last_entry_t_nolock(st));
-
-    rrddim_foreach_read(rd1, st) {
-        RRDDIM *rd = onewayalloc_memdupz(owa, rd1, rd1->memsize);
-        rd->id = onewayalloc_strdupz(owa, rd1->id);
-        rd->name = onewayalloc_strdupz(owa, rd1->name);
-        rd->state = onewayalloc_memdupz(owa, rd1->state, sizeof(*rd->state));
-        rd->next = (*param_list)->rd;
-        (*param_list)->rd = rd;
-    }
-
-    rrdset_unlock(st);
-}
+#include "database/storage_engine.h"
 
 void rrd_stats_api_v1_chart(RRDSET *st, BUFFER *wb) {
     rrdset2json(st, wb, NULL, NULL, 0);
@@ -152,24 +58,45 @@ void rrdr_buffer_print_format(BUFFER *wb, uint32_t format)  {
 int rrdset2value_api_v1(
           RRDSET *st
         , BUFFER *wb
-        , calculated_number *n
+        , NETDATA_DOUBLE *n
         , const char *dimensions
-        , long points
-        , long long after
-        , long long before
-        , int group_method
-        , long group_time
+        , size_t points
+        , time_t after
+        , time_t before
+        , RRDR_GROUPING group_method
+        , const char *group_options
+        , time_t resampling_time
         , uint32_t options
         , time_t *db_after
         , time_t *db_before
+        , size_t *db_points_read
+        , size_t *db_points_per_tier
+        , size_t *result_points_generated
         , int *value_is_null
-        , int timeout
+        , NETDATA_DOUBLE *anomaly_rate
+        , time_t timeout
+        , size_t tier
+        , QUERY_SOURCE query_source
+        , STORAGE_PRIORITY priority
 ) {
     int ret = HTTP_RESP_INTERNAL_SERVER_ERROR;
 
     ONEWAYALLOC *owa = onewayalloc_create(0);
-
-    RRDR *r = rrd2rrdr(owa, st, points, after, before, group_method, group_time, options, dimensions, NULL, timeout);
+    RRDR *r = rrd2rrdr_legacy(
+            owa,
+            st,
+            points,
+            after,
+            before,
+            group_method,
+            resampling_time,
+            options,
+            dimensions,
+            group_options,
+            timeout,
+            tier,
+            query_source,
+            priority);
 
     if(!r) {
         if(value_is_null) *value_is_null = 1;
@@ -177,9 +104,18 @@ int rrdset2value_api_v1(
         goto cleanup;
     }
 
-    if(rrdr_rows(r) == 0) {
-        rrdr_free(owa, r);
+    if(db_points_read)
+        *db_points_read += r->internal.db_points_read;
 
+    if(db_points_per_tier) {
+        for(size_t t = 0; t < storage_tiers ;t++)
+            db_points_per_tier[t] += r->internal.tier_points_read[t];
+    }
+
+    if(result_points_generated)
+        *result_points_generated += r->internal.result_points_generated;
+
+    if(rrdr_rows(r) == 0) {
         if(db_after)  *db_after  = 0;
         if(db_before) *db_before = 0;
         if(value_is_null) *value_is_null = 1;
@@ -198,47 +134,19 @@ int rrdset2value_api_v1(
     if(db_after)  *db_after  = r->after;
     if(db_before) *db_before = r->before;
 
-    long i = (!(options & RRDR_OPTION_REVERSED))?rrdr_rows(r) - 1:0;
-    *n = rrdr2value(r, i, options, value_is_null, NULL);
+    long i = (!(options & RRDR_OPTION_REVERSED))?(long)rrdr_rows(r) - 1:0;
+    *n = rrdr2value(r, i, options, value_is_null, anomaly_rate);
     ret = HTTP_RESP_OK;
 
 cleanup:
-    if(r) rrdr_free(owa, r);
+    rrdr_free(owa, r);
     onewayalloc_destroy(owa);
     return ret;
 }
 
-int rrdset2anything_api_v1(
-          ONEWAYALLOC *owa
-        , RRDSET *st
-        , QUERY_PARAMS *query_params
-        , BUFFER *dimensions
-        , uint32_t format
-        , long points
-        , long long after
-        , long long before
-        , int group_method
-        , long group_time
-        , uint32_t options
-        , time_t *latest_timestamp
-)
-{
-    BUFFER *wb = query_params->wb;
-    if (query_params->context_param_list && !(query_params->context_param_list->flags & CONTEXT_FLAGS_ARCHIVE))
-        st->last_accessed_time = now_realtime_sec();
+int data_query_execute(ONEWAYALLOC *owa, BUFFER *wb, QUERY_TARGET *qt, time_t *latest_timestamp) {
 
-    RRDR *r = rrd2rrdr(
-        owa,
-        st,
-        points,
-        after,
-        before,
-        group_method,
-        group_time,
-        options,
-        dimensions ? buffer_tostring(dimensions) : NULL,
-        query_params->context_param_list,
-        query_params->timeout);
+    RRDR *r = rrd2rrdr(owa, qt);
     if(!r) {
         buffer_strcat(wb, "Cannot generate output with these parameters on this chart.");
         return HTTP_RESP_INTERNAL_SERVER_ERROR;
@@ -249,11 +157,6 @@ int rrdset2anything_api_v1(
         return HTTP_RESP_BACKEND_FETCH_FAILED;
     }
 
-    if (st && st->state && st->state->is_ar_chart)
-        ml_process_rrdr(r, query_params->max_anomaly_rates);
-
-    RRDDIM *temp_rd = query_params->context_param_list ? query_params->context_param_list->rd : NULL;
-
     if(r->result_options & RRDR_RESULT_OPTION_RELATIVE)
         buffer_no_cacheable(wb);
     else if(r->result_options & RRDR_RESULT_OPTION_ABSOLUTE)
@@ -262,85 +165,89 @@ int rrdset2anything_api_v1(
     if(latest_timestamp && rrdr_rows(r) > 0)
         *latest_timestamp = r->before;
 
+    DATASOURCE_FORMAT format = qt->request.format;
+    RRDR_OPTIONS options = qt->request.options;
+    RRDR_GROUPING group_method = qt->request.group_method;
+
     switch(format) {
     case DATASOURCE_SSV:
         if(options & RRDR_OPTION_JSON_WRAP) {
             wb->contenttype = CT_APPLICATION_JSON;
-            rrdr_json_wrapper_begin(r, wb, format, options, 1, query_params);
-            rrdr2ssv(r, wb, options, "", " ", "", temp_rd);
+            rrdr_json_wrapper_begin(r, wb, format, options, 1, group_method);
+            rrdr2ssv(r, wb, options, "", " ", "");
             rrdr_json_wrapper_end(r, wb, format, options, 1);
         }
         else {
             wb->contenttype = CT_TEXT_PLAIN;
-            rrdr2ssv(r, wb, options, "", " ", "", temp_rd);
+            rrdr2ssv(r, wb, options, "", " ", "");
         }
         break;
 
     case DATASOURCE_SSV_COMMA:
         if(options & RRDR_OPTION_JSON_WRAP) {
             wb->contenttype = CT_APPLICATION_JSON;
-            rrdr_json_wrapper_begin(r, wb, format, options, 1, query_params);
-            rrdr2ssv(r, wb, options, "", ",", "", temp_rd);
+            rrdr_json_wrapper_begin(r, wb, format, options, 1, group_method);
+            rrdr2ssv(r, wb, options, "", ",", "");
             rrdr_json_wrapper_end(r, wb, format, options, 1);
         }
         else {
             wb->contenttype = CT_TEXT_PLAIN;
-            rrdr2ssv(r, wb, options, "", ",", "", temp_rd);
+            rrdr2ssv(r, wb, options, "", ",", "");
         }
         break;
 
     case DATASOURCE_JS_ARRAY:
         if(options & RRDR_OPTION_JSON_WRAP) {
             wb->contenttype = CT_APPLICATION_JSON;
-            rrdr_json_wrapper_begin(r, wb, format, options, 0, query_params);
-            rrdr2ssv(r, wb, options, "[", ",", "]", temp_rd);
+            rrdr_json_wrapper_begin(r, wb, format, options, 0, group_method);
+            rrdr2ssv(r, wb, options, "[", ",", "]");
             rrdr_json_wrapper_end(r, wb, format, options, 0);
         }
         else {
             wb->contenttype = CT_APPLICATION_JSON;
-            rrdr2ssv(r, wb, options, "[", ",", "]", temp_rd);
+            rrdr2ssv(r, wb, options, "[", ",", "]");
         }
         break;
 
     case DATASOURCE_CSV:
         if(options & RRDR_OPTION_JSON_WRAP) {
             wb->contenttype = CT_APPLICATION_JSON;
-            rrdr_json_wrapper_begin(r, wb, format, options, 1, query_params);
-            rrdr2csv(r, wb, format, options, "", ",", "\\n", "", temp_rd);
+            rrdr_json_wrapper_begin(r, wb, format, options, 1, group_method);
+            rrdr2csv(r, wb, format, options, "", ",", "\\n", "");
             rrdr_json_wrapper_end(r, wb, format, options, 1);
         }
         else {
             wb->contenttype = CT_TEXT_PLAIN;
-            rrdr2csv(r, wb, format, options, "", ",", "\r\n", "", temp_rd);
+            rrdr2csv(r, wb, format, options, "", ",", "\r\n", "");
         }
         break;
 
     case DATASOURCE_CSV_MARKDOWN:
         if(options & RRDR_OPTION_JSON_WRAP) {
             wb->contenttype = CT_APPLICATION_JSON;
-            rrdr_json_wrapper_begin(r, wb, format, options, 1, query_params);
-            rrdr2csv(r, wb, format, options, "", "|", "\\n", "", temp_rd);
+            rrdr_json_wrapper_begin(r, wb, format, options, 1, group_method);
+            rrdr2csv(r, wb, format, options, "", "|", "\\n", "");
             rrdr_json_wrapper_end(r, wb, format, options, 1);
         }
         else {
             wb->contenttype = CT_TEXT_PLAIN;
-            rrdr2csv(r, wb, format, options, "", "|", "\r\n", "", temp_rd);
+            rrdr2csv(r, wb, format, options, "", "|", "\r\n", "");
         }
         break;
 
     case DATASOURCE_CSV_JSON_ARRAY:
         wb->contenttype = CT_APPLICATION_JSON;
         if(options & RRDR_OPTION_JSON_WRAP) {
-            rrdr_json_wrapper_begin(r, wb, format, options, 0, query_params);
+            rrdr_json_wrapper_begin(r, wb, format, options, 0, group_method);
             buffer_strcat(wb, "[\n");
-            rrdr2csv(r, wb, format, options + RRDR_OPTION_LABEL_QUOTES, "[", ",", "]", ",\n", temp_rd);
+            rrdr2csv(r, wb, format, options + RRDR_OPTION_LABEL_QUOTES, "[", ",", "]", ",\n");
             buffer_strcat(wb, "\n]");
             rrdr_json_wrapper_end(r, wb, format, options, 0);
         }
         else {
             wb->contenttype = CT_APPLICATION_JSON;
             buffer_strcat(wb, "[\n");
-            rrdr2csv(r, wb, format, options + RRDR_OPTION_LABEL_QUOTES, "[", ",", "]", ",\n", temp_rd);
+            rrdr2csv(r, wb, format, options + RRDR_OPTION_LABEL_QUOTES, "[", ",", "]", ",\n");
             buffer_strcat(wb, "\n]");
         }
         break;
@@ -348,29 +255,29 @@ int rrdset2anything_api_v1(
     case DATASOURCE_TSV:
         if(options & RRDR_OPTION_JSON_WRAP) {
             wb->contenttype = CT_APPLICATION_JSON;
-            rrdr_json_wrapper_begin(r, wb, format, options, 1, query_params);
-            rrdr2csv(r, wb, format, options, "", "\t", "\\n", "", temp_rd);
+            rrdr_json_wrapper_begin(r, wb, format, options, 1, group_method);
+            rrdr2csv(r, wb, format, options, "", "\t", "\\n", "");
             rrdr_json_wrapper_end(r, wb, format, options, 1);
         }
         else {
             wb->contenttype = CT_TEXT_PLAIN;
-            rrdr2csv(r, wb, format, options, "", "\t", "\r\n", "", temp_rd);
+            rrdr2csv(r, wb, format, options, "", "\t", "\r\n", "");
         }
         break;
 
     case DATASOURCE_HTML:
         if(options & RRDR_OPTION_JSON_WRAP) {
             wb->contenttype = CT_APPLICATION_JSON;
-            rrdr_json_wrapper_begin(r, wb, format, options, 1, query_params);
+            rrdr_json_wrapper_begin(r, wb, format, options, 1, group_method);
             buffer_strcat(wb, "<html>\\n<center>\\n<table border=\\\"0\\\" cellpadding=\\\"5\\\" cellspacing=\\\"5\\\">\\n");
-            rrdr2csv(r, wb, format, options, "<tr><td>", "</td><td>", "</td></tr>\\n", "", temp_rd);
+            rrdr2csv(r, wb, format, options, "<tr><td>", "</td><td>", "</td></tr>\\n", "");
             buffer_strcat(wb, "</table>\\n</center>\\n</html>\\n");
             rrdr_json_wrapper_end(r, wb, format, options, 1);
         }
         else {
             wb->contenttype = CT_TEXT_HTML;
             buffer_strcat(wb, "<html>\n<center>\n<table border=\"0\" cellpadding=\"5\" cellspacing=\"5\">\n");
-            rrdr2csv(r, wb, format, options, "<tr><td>", "</td><td>", "</td></tr>\n", "", temp_rd);
+            rrdr2csv(r, wb, format, options, "<tr><td>", "</td><td>", "</td></tr>\n", "");
             buffer_strcat(wb, "</table>\n</center>\n</html>\n");
         }
         break;
@@ -379,9 +286,9 @@ int rrdset2anything_api_v1(
         wb->contenttype = CT_APPLICATION_X_JAVASCRIPT;
 
         if(options & RRDR_OPTION_JSON_WRAP)
-            rrdr_json_wrapper_begin(r, wb, format, options, 0, query_params);
+            rrdr_json_wrapper_begin(r, wb, format, options, 0, group_method);
 
-        rrdr2json(r, wb, options, 1, query_params->context_param_list);
+        rrdr2json(r, wb, options, 1);
 
         if(options & RRDR_OPTION_JSON_WRAP)
             rrdr_json_wrapper_end(r, wb, format, options, 0);
@@ -391,9 +298,9 @@ int rrdset2anything_api_v1(
         wb->contenttype = CT_APPLICATION_JSON;
 
         if(options & RRDR_OPTION_JSON_WRAP)
-            rrdr_json_wrapper_begin(r, wb, format, options, 0, query_params);
+            rrdr_json_wrapper_begin(r, wb, format, options, 0, group_method);
 
-        rrdr2json(r, wb, options, 1, query_params->context_param_list);
+        rrdr2json(r, wb, options, 1);
 
         if(options & RRDR_OPTION_JSON_WRAP)
             rrdr_json_wrapper_end(r, wb, format, options, 0);
@@ -402,9 +309,9 @@ int rrdset2anything_api_v1(
     case DATASOURCE_JSONP:
         wb->contenttype = CT_APPLICATION_X_JAVASCRIPT;
         if(options & RRDR_OPTION_JSON_WRAP)
-            rrdr_json_wrapper_begin(r, wb, format, options, 0, query_params);
+            rrdr_json_wrapper_begin(r, wb, format, options, 0, group_method);
 
-        rrdr2json(r, wb, options, 0, query_params->context_param_list);
+        rrdr2json(r, wb, options, 0);
 
         if(options & RRDR_OPTION_JSON_WRAP)
             rrdr_json_wrapper_end(r, wb, format, options, 0);
@@ -415,12 +322,17 @@ int rrdset2anything_api_v1(
         wb->contenttype = CT_APPLICATION_JSON;
 
         if(options & RRDR_OPTION_JSON_WRAP)
-            rrdr_json_wrapper_begin(r, wb, format, options, 0, query_params);
+            rrdr_json_wrapper_begin(r, wb, format, options, 0, group_method);
 
-        rrdr2json(r, wb, options, 0, query_params->context_param_list);
+        rrdr2json(r, wb, options, 0);
 
-        if(options & RRDR_OPTION_JSON_WRAP)
+        if(options & RRDR_OPTION_JSON_WRAP) {
+            if(options & RRDR_OPTION_RETURN_JWAR) {
+                rrdr_json_wrapper_anomaly_rates(r, wb, format, options, 0);
+                rrdr2json(r, wb, options | RRDR_OPTION_INTERNAL_AR, 0);
+            }
             rrdr_json_wrapper_end(r, wb, format, options, 0);
+        }
         break;
     }
 
